<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyPaymentWebService.php';
/**
 * AdminThreeByFourOneyConfirmationController class
 * Admin controller focused on order confirmation.
 * 
 * @version 3.0.0
 */
class AdminThreeByFourOneyConfirmationController extends ModuleAdminController
{
    protected $payment_web_service;
    protected $utility;
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return mixed
     */
    public function __construct()
    {
        $this->utility = new ThreeByFourOneyUtility();
        $this->payment_web_service = new ThreeByFourOneyPaymentWebService();
    }

    /**
     * sendConfirmation function
     * Handle confirmation request between Prestashop and Oney's API.
     *
     * @version 3.0.0
     * @param int $idOrder
     * @return array
     */
    public function sendConfirmation($idOrder)
    {
        $order = new Order($idOrder);
        $shopId = (int)$order->id_shop;
        $environment = (string)Configuration::get('TBFONEY_ENVIRONMENT', null, null, $shopId, false);
        $reference = (string)$order->reference;
        $amount = (float)$order->total_paid;
        $billingAddress = new Address((int)$order->id_address_invoice);
        $countryCode = (string)Country::getIsoById((int)$billingAddress->id_country);
        $logsConfig = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS', null, null, $shopId, false);

        $requestBody = [
            'language_code' => strtoupper((string)$countryCode),
            'merchant_request_id' => date('YmdHis') . '-' . (string)$idOrder,
            'payment' => [
                'payment_amount' => number_format((float)$amount, 2, '.', '')
            ]
        ];
        $request = $this->payment_web_service->requestAction($environment, strtoupper($countryCode), 'confirm', $reference, $requestBody, $shopId);

        if ($logsConfig === 'yes')
        {
            $log = new ThreeByFourOneyLogger();
            $log->debug('[PAYMENT API] SEND CONFIRMATION REQUEST BODY: ' . json_encode($requestBody));
        }
        return $request;
    }
}