<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyBusinessTransactionsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyMarketingWebService.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

/**
 * AdminThreeByFourOneyController class
 * Admin controller several uses (mainly ajax manipulations).
 * 
 * @version 3.0.0
 */
class AdminThreeByFourOneyController extends ModuleAdminController
{
    protected $business_transactions;
    protected $marketing_web_service;
    protected $utility;
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return mixed
     */
    public function __construct()
    {
        $this->utility = new ThreeByFourOneyUtility();
        $this->business_transactions = new ThreeByFourOneyBusinessTransactionsModel();
        $this->marketing_web_service = new ThreeByFourOneyMarketingWebService();
        
        return parent::__construct();
    }
    /**
     * ajaxProcessUpdateEnvironment function
     * Update configuration based on administration panel environment select
     *
     * @version 3.0.0|3.0.4
     * @return void
     */
    public function ajaxProcessUpdateThreeByFourOneyEnvironment()
    {
        $environment = Tools::getValue('environment');
        (string)Configuration::updateValue('TBFONEY_ENVIRONMENT', $environment);
    }

    /**
     * ajaxProcessRequestBusinessTransactions function
     * Request Business transactions on administration panel for module configuration.
     *
     * @version 3.0.0|3.0.4|3.0.5
     * @return void
     */
    public function ajaxProcessRequestThreeByFourOneyBusinessTransactions()
    {
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $ptIsSpecificCountryValidatedConfig = (bool)Configuration::get('TBFONEY_PT_IS_CONFIG_VALIDATED');
        $shopId = (int)Context::getContext()->shop->id;
        // 1. Delete all recorded business transactions
        $this->business_transactions->deleteBusinessTransactions($shopId);
        // 2. Request business transaction from  API
        $countries = $this->utility->countriesMapper();
        $businessTransactionType = ["PNFCB", "Short", "ShortCredit", "Long"];

        foreach ($countries as $country)
        {
            if (($country['iso_code'] === 'PT') && $ptIsSpecificCountryValidatedConfig === false) continue;
            $businessTransactionsRecords = $this->business_transactions->getBusinessTransactions($country['iso_code'], $shopId);
            $businessTransactionsWebService = $this->marketing_web_service->requestBusinessTransactionsCatalog($environmentConfig, $country['iso_code']);
            if ($businessTransactionsWebService != null)
            {
                foreach ($businessTransactionsWebService as $bt)
                {
                    if (in_array((string)$bt->business_transaction_type, $businessTransactionType))
                    {  
                        $data = [];
                        $data['business_transaction_code'] = (string)$bt->business_transaction_code;
                        $data['business_transaction_version'] = (int)$bt->business_transaction_version;
                        $data['business_transaction_type'] = (string)$bt->business_transaction_type;
                        $data['customer_label'] = (string)$bt->customer_label;
                        $data['short_label'] = (string)$bt->short_label;
                        $data['long_label'] = (string)$bt->long_label;
                        $franceOverseasTerritoriesCodes = ['GP', 'GF', 'RE', 'MQ', 'YT', 'NC', 'PF', 'BL', 'MF', 'PM', 'WF'];
                        if (($country['iso_code'] == 'FR' || in_array($country['iso_code'], $franceOverseasTerritoriesCodes)) && $bt->maximum_number_of_instalments > 1)
                        {
                            $data['minimum_number_of_instalments'] = (int)$bt->maximum_number_of_instalments + 1;
                            $data['maximum_number_of_instalments'] = (int)$bt->maximum_number_of_instalments + 1;
                        } else {
                            $data['minimum_number_of_instalments'] = (int)$bt->maximum_number_of_instalments;
                            $data['maximum_number_of_instalments'] =(int)$bt->maximum_number_of_instalments;
                        }
                        $data['minimum_selling_price'] = (float)$bt->minimum_selling_price;
                        $data['maximum_selling_price'] = (float)$bt->maximum_selling_price;
                        $data['discount_rate'] = (float)$bt->discount_rate;
                        $data['choosable_number_of_instalments'] = (int)$bt->choosable_number_of_instalments;
                        $data['free_business_transaction'] = (int)$bt->free_business_transaction;
                        $data['is_active'] = (int)false;
                        $data['country_code'] = (string)$country['iso_code'];
                        $data['shop_id'] = (int)$shopId;
                        $this->business_transactions->insertBusinessTransactions($data);
                    }
                }
            }
            
            if ($businessTransactionsRecords != null)
            {
                $this->ajaxDie(json_encode($businessTransactionsRecords));
            }
        }
    }

    /**
     * ajaxProcessWizardNavigateToSelectedtStep function
     * ajaxProcessWizardNavigateToSelectedtStep defines behaviours expected when the user clicks on header nav link.
     * This action modify TBFONEY_WIZARD_STEP and TBFONEY_IS_TERMS_ACCEPTED configuration values
     *
     * @version 3.1.11
     * @return void
     */
    protected function ajaxProcessWizardNavigateToSelectedtStep()
    {
        $selectedStep = Tools::getValue('selectedStep');
        Configuration::updateValue('TBFONEY_WIZARD_STEP', $selectedStep);
    }

    /**
     * ajaxProcessDeleteLegalFile function
     * Delete Portugal legal file.
     *
     * @version 3.0.0|3.0.3|3.1.0
     * @return void
     */
    public function ajaxProcessDeleteLegalFile()
    {
        $shopId = (int)Context::getContext()->shop->id;
        $files = scandir(_PS_MODULE_DIR_ . 'threebyfouroney/uploads/documents/pt/');
        foreach($files as $file)
        {
            if ($file == "index.php" || $file == "." || $file == "..") continue;
            unlink(_PS_MODULE_DIR_ . 'threebyfouroney/uploads/documents/pt/' . $file);
        }
        Configuration::updateValue('TBFONEY_PT_LEGAL_DOCUMENT', null);
        (bool)Configuration::updateValue('TBFONEY_PT_IS_CONFIG_VALIDATED', 0, null, 0, $shopId);
    }
}