<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCarriersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyOrdersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyPaymentWebService.php';
/**
 * ThreeByFourOneyPaymentModuleFrontController class
 * Front controller for payment requests.
 * 
 * @version 3.0.0
 */
class ThreeByFourOneyPaymentModuleFrontController extends ModuleFrontController
{
    protected $carriers;
    protected $utility;
    protected $payment_web_service;
    protected $orders;
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return void
     */
    public function __construct()
    {
        $this->carriers = new ThreeByFourOneyCarriersModel();
        $this->utility = new ThreeByFourOneyUtility();
        $this->payment_web_service = new ThreeByFourOneyPaymentWebService();
        $this->orders = new ThreeByFourOneyOrdersModel();
        parent::__construct();
    }

    /**
     * shippingAddressesManagement function
     * Shipping addresses management based on end customer's configuration and transportations choises.
     * 
     * @version 3.0.0
     * @return array
     */
    protected function shippingAddressesManagement()
    {
        $shippingAddress = new Address((int)$this->cartsManagement()->id_address_delivery);
        $billingAddress = new Address((int)$this->cartsManagement()->id_address_invoice);
        // Shipping adresses conditioned by address type
        $Addresstype = (int)$this->carriersManagement()['address_type'];
        $storeId = (int)$this->carriersManagement()['store_id'];
        $langId = (int)Context::getContext()->language->id;
        if ($storeId != 0)
        {
            $storeAddress = $this->carriers->getStoreById($storeId);
            $storeCountry = new Country($storeAddress['id_country'], $langId);
        }
        switch ($Addresstype)
        {
            case 1;
                // Return store address information
                $data = $this->utility->splitAddresses($storeAddress['address1'] . ' ' . $storeAddress['address2']);
                $data['pickup_point_name'] = $storeAddress['name'];
                $data['postal_code'] = $storeAddress['postcode'];
                $data['municipality'] = $storeAddress['city'];
                $data['country_code'] = $storeCountry->iso_code;
                $data['country_label'] = $storeCountry->name;
                $data['phone_number'] = $storeAddress['phone'];
                return $data;
                break;
            case 2;
            case 3;
                // Return Third party and airports information
                $data = $this->utility->splitAddresses($billingAddress->address1 . ' ' . $billingAddress->address2);
                $data['postal_code'] = $billingAddress->postcode;
                $data['municipality'] = $billingAddress->city;
                $data['country_code'] = Country::getIsoById((int)$billingAddress->id_country);
                $data['country_label'] = $billingAddress->country;
                $data['phone_number'] = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
                return $data;
                break;
            case 4;
                // Return billing address information
                $data = $this->utility->splitAddresses($billingAddress->address1 . ' ' . $billingAddress->address2);
                $data['postal_code'] = $billingAddress->postcode;
                $data['municipality'] = $billingAddress->city;
                $data['country_code'] = Country::getIsoById($billingAddress->id_country);
                $data['country_label'] = $billingAddress->country;
                $data['phone_number'] = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
                return $data;
                break;
            case 5;
            case 6;
                // Return shipping address information
                $data = $this->utility->splitAddresses($shippingAddress->address1 . ' ' . $shippingAddress->address2);
                $data['postal_code'] = $shippingAddress->postcode;
                $data['municipality'] = $shippingAddress->city;
                $data['country_code'] = Country::getIsoById($shippingAddress->id_country);
                $data['country_label'] = $shippingAddress->country;
                $data['phone_number'] = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
                return $data;
                break;
            default;
                // Default: return billing address information
                $data = $this->utility->splitAddresses($billingAddress->address1 . ' ' . $billingAddress->address2);
                $data['postal_code'] = $billingAddress->postcode;
                $data['municipality'] = $billingAddress->city;
                $data['country_code'] = Country::getIsoById($billingAddress->id_country);
                $data['country_label'] = $billingAddress->country;
                $data['phone_number'] = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
                return $data;
        }
        
    }

    /**
     * billingAddressesManagement function
     * Billing addresses management based on end customer's configuration and transportations choises.
     * 
     * @version 3.0.0
     * @return array
     */
    protected function billingAddressesManagement()
    {
        $billingAddress = new Address($this->cartsManagement()->id_address_invoice);
        $data = $this->utility->splitAddresses($billingAddress->address1 . ' ' . $billingAddress->address2);
        $data['postal_code'] = $billingAddress->postcode;
        $data['municipality'] = $billingAddress->city;
        $data['country_code'] = Country::getIsoById((int)$billingAddress->id_country);
        $data['country_label'] = $billingAddress->country;
        $data['phone_number'] = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
        return $data;
    }

    /**
     * carriersManagement function
     * Carriers management based on module's administration panel configuration.
     * 
     * @version 3.0.0|3.0.5
     * @return array
     */
    protected function carriersManagement()
    {
        $shopId = (int)Tools::getValue('sid');
        $carriers = $this->carriers->getCarriersWithParams($this->cartsManagement()->id_carrier, $shopId);
        foreach ($carriers as $carrier)
        {
            $data = [
                'address_type'          => $carrier['address_type'],
                'delivery_mode'         => $carrier['delivery_mode'],
                'delivery_option'       => $carrier['delivery_option'],
                'priority_delivery'     => $carrier['priority_delivery'],
                'store_id'              => $carrier['store_id'],
                'shop_id'               => $shopId
            ];
            return $data;
        }

        // return default values for virtual products
        return array(
            'address_type'          => 6,
            'delivery_mode'         => 5,
            'delivery_option'       => 1,
            'priority_delivery'     => 3,
            'store_id'              => 0,
            'shop_id'               => $shopId
        );
    }

    /**
     * cartsManagement function
     * Return current customer's cart's information for further manipulation.
     * 
     * @version 3.0.0
     * @return object
     */
    protected function cartsManagement()
    {
        $data = Context::getContext()->cart;
        return $data;
    }

    /**
     * customersManagement function
     * Return current customer's information for further manipulation.
     * 
     * @version 3.0.0
     * @return object
     */
    protected function customersManagement()
    {
        $data = new Customer($this->cartsManagement()->id_customer);
        return $data;
    }

    /**
     * ordersManagement function
     * Handle order's creation (prerequisite for payment and order manipulation)
     *
     * @version 3.0.0
     * @param object $customerCart
     * @param object $customerInfo
     * @param int $total
     * @return Order
     */
    protected function ordersManagement($customerCart, $customerInfo, $total)
    {
        $orderStateId = (int)Configuration::get('TBFONEY_OS_PENDING');
        $this->module->validateOrder($customerCart->id, $orderStateId, $total, $this->module->displayName, null, [], null, false, $customerInfo->secure_key);
        $currentOrderId = Order::getOrderByCartId((int)$customerCart->id);
        $currentOrder = new Order((int)$currentOrderId);
        $reference = $currentOrder->reference;
        $shopId = $currentOrder->id_shop;
        // insert order in ThreeByFourOneyOrders 
        $data = [];
        $data['id_order'] = (int)$currentOrderId;
        $data['reference'] = $reference;
        $data['amount'] = (float)$total;
        $data['country_code'] = (string)$this->utility->customerCountryConfiguration();
        $data['shop_id'] = (int)$shopId;
        $data['date_add'] = date('Y-m-d H:i:s');
        $this->orders->insertOrder($data);

        return $currentOrder;
    }

    /**
     * productsManagement function
     * Handle cart's product(s) management for request body manipulation.
     * 
     * @version 3.0.0
     * @return array
     */
    protected function productsManagement()
    {
        $cart = $this->cartsManagement();
        $carrier = new Carrier($cart->id_carrier, $cart->id_lang);
        $products = $cart->getProducts();
        $productQuantity = 0;
        $transportPrice = $cart->getTotalShippingCost();
        $itemList = [];
        $productMostExpensiveKey = 0;
        $productPriceMostExpensive = 0;
        foreach ($products as $key => $product)
        {
            $productPrice = (isset($product['price_with_reduction']) ? number_format((float)$product['price_with_reduction'], 2, '.', '') : number_format((float)$product['price_wt'], 2, '.', ''));
            if ($productPrice > $productPriceMostExpensive) {
                $productPriceMostExpensive = $productPrice;
                $productMostExpensiveKey = $key;
            }

            $productData = [];
            $productData['is_main_item'] = 0;
            $productData['category_code'] = (int)Configuration::get('TBFONEY_STORE_CATEGORY');
            $productData['label'] = (string)$product['name'];
            $productData['item_external_code'] = (string)$product['reference'];
            $productData['quantity'] = (int)$product['quantity'];
            $productQuantity += (int)$product['quantity'];
            $productData['price'] = number_format((float)$productPrice, 2, '.', '');
            $itemList[] = $productData;
        }
        $itemList[$productMostExpensiveKey]['is_main_item'] = 1;

        $transportData = [];
        $transportData['is_main_item'] = 0;
        $transportData['category_code'] = 11;
        $transportData['label'] = $carrier->id ? $carrier->name : "Virtual";
        $transportData['item_external_code'] = 'TRANSPORT';
        $transportData['quantity'] = 1;
        $productQuantity += $transportData['quantity'];
        $transportData['price'] = number_format($transportPrice, 2, '.', '');
        
        $itemList[] = $transportData;
        $itemList[]['total_quantities'] = $productQuantity;

        return $itemList;
    }

    /**
     * postProcess function
     *
     * Based on above functions, consolidate request body prerequisite for One'y payment request.
     * @return mixed
     */
    public function postProcess()
    {
        $businessTransactionCode = (string)Tools::getValue('business_transaction_code');
        $logsConfig = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS');

        if (isset($businessTransactionCode) && !empty($businessTransactionCode) && ($businessTransactionCode != null))
        {
            $environment = (string)$this->utility->credentialsManagement()['environment'];
            $countryCode = (string)$this->utility->customerCountryConfiguration();

            $customerCarrier = $this->carriersManagement();
            $customerCart = $this->cartsManagement();
            $customerCurrency = Context::getContext()->currency->iso_code;
            $customerInfo = $this->customersManagement();
            $total = (float)$customerCart->getOrderTotal();
            $customerOrder = $this->ordersManagement($customerCart, $customerInfo, $total);
            $merchantCredentials = $this->utility->credentialsManagement();
            $shippingAddress = $this->shippingAddressesManagement();
            $billingAddress = $this->billingAddressesManagement();
            $cartProducts = $this->productsManagement();
            
            $productsArray = array_reduce($cartProducts, function($array, $item){
                $array[] = $item;
                return $array;
            });

            $requestBody = [
                'merchant_guid' => (string)$merchantCredentials['merchant_guid'],
                'psp_guid' => (string)$merchantCredentials['psp_guid'],
                'merchant_request_id' => (date('YmdHis') . '_' . $customerCart->id),
                'purchase' => [
                    'external_reference_type' => 'CMDE',
                    'external_reference' => $customerOrder->reference,
                    'purchase_amount' => number_format($total, 2, '.', ''),
                    'currency_code' => $customerCurrency,
                    'delivery' => [
                        'delivery_date' => date('Y-m-d'),
                        'delivery_mode_code' => (int)$customerCarrier['delivery_mode'],
                        'delivery_option' => (int)$customerCarrier['delivery_option'],
                        'priority_delivery_code' => (int)$customerCarrier['priority_delivery'],
                        'address_type' => (int)$customerCarrier['address_type'],
                        'recipient' => [
                            'surname' => (string)$customerInfo->lastname,
                            'first_name' => (string)$customerInfo->firstname,
                            'phone_number' => (string)$billingAddress['phone_number']
                        ],
                        'delivery_address' => [
                            'pickup_point_name' => (isset($shippingAddress['pickup_point_name']) && !empty($shippingAddress['pickup_point_name'])) ? (string)$shippingAddress['pickup_point_name'] : '',
                            'line1' => (string)(isset($shippingAddress[0]) && !empty($shippingAddress[0])) ? $shippingAddress[0] : '',
                            'line2' => (string)(isset($shippingAddress[1]) && !empty($shippingAddress[1])) ? $shippingAddress[1] : '',
                            'line3' => (string)(isset($shippingAddress[2]) && !empty($shippingAddress[2])) ? $shippingAddress[2] : '',
                            'line4' => (string)(isset($shippingAddress[3]) && !empty($shippingAddress[3])) ? $shippingAddress[3] : '',
                            'line5' => (string)(isset($shippingAddress[4]) && !empty($shippingAddress[4])) ? $shippingAddress[4] : '',
                            'postal_code' => (string)$shippingAddress['postal_code'],
                            'municipality' => (string)$shippingAddress['municipality'],
                            'country_code' => (string)$this->utility->getConfigByLanguage('iso_alpha3', strtolower($countryCode)),
                            'country_label' => (string)$shippingAddress['country_label']
                        ]
                    ],
                    'number_of_items' => (int)end($productsArray)['total_quantities'],
                    'item_list' => array_slice($productsArray, 0, -1)
                ],
                'customer' => [
                    'customer_external_code' => (string)($customerInfo->id),
                    'language_code' => strtoupper($countryCode),
                    'identity' => [
                        'person_type' => (int)(isset($customerInfo->siret) && !empty($customerInfo->siret)) ? 1 : 2,
                        'honorific_code' => (int)$customerInfo->id_gender,
                        'birth_name' => (string)$customerInfo->lastname,
                        'first_name' => (string)$customerInfo->firstname
                    ],
                    'contact_details' => [
                        'landline_number' => (string)$shippingAddress['phone_number'],
                        'mobile_phone_number' => (string)$billingAddress['phone_number'],
                        'email_address' => (string)$customerInfo->email
                    ],
                    'customer_address' => [
                        'line1' => (string)(isset($billingAddress[0]) && !empty($billingAddress[0])) ? $billingAddress[0] : '',
                        'line2' => (string)(isset($billingAddress[1]) && !empty($billingAddress[1])) ? $billingAddress[1] : '',
                        'line3' => (string)(isset($billingAddress[2]) && !empty($billingAddress[2])) ? $billingAddress[2] : '',
                        'line4' => (string)(isset($billingAddress[3]) && !empty($billingAddress[3])) ? $billingAddress[3] : '',
                        'line5' => (string)(isset($billingAddress[4]) && !empty($billingAddress[4])) ? $billingAddress[4] : '',
                        'postal_code' => (string)$billingAddress['postal_code'],
                        'municipality' => (string)$billingAddress['municipality'],
                        'country_code' => (string)$this->utility->getConfigByLanguage('iso_alpha3', strtolower($countryCode)),
                        'country_label' => (string)$billingAddress['country_label']
                    ]
                ],
                'payment' => [
                    'merchant_guid' => (string)$merchantCredentials['merchant_guid'],
                    'payment_amount' => number_format($customerCart->getOrderTotal(), 2, '.', ''),
                    'currency_code' => $customerCurrency,
                    'business_transaction' => [
                        'code' => $businessTransactionCode
                    ]
                ],
                'navigation' => [
                    'success_url' => $this->context->link->getModuleLink($this->module->name, 'success', ['id' => $customerCart->id, 'reference' => $customerOrder->reference, 'amount' => number_format((float)$customerCart->getOrderTotal(), 2, '.', ''), 'iso_code' => $countryCode], true),
                    'fail_url' => $this->context->link->getModuleLink($this->module->name, 'failure', ['id' => $customerCart->id, 'reference' => $customerOrder->reference, 'iso_code' => $countryCode], true),
                    'server_response_url' => $this->context->link->getModuleLink($this->module->name, 'updateorderstatus', ['reference' => $customerOrder->reference, 'iso_code' => $countryCode], true)
                ],
                'merchant_context' => md5((date('YmdHis') . $customerCart->id . $customerOrder->reference)),
                'psp_context' => md5((date('YmdHis') . $customerCart->id . $customerOrder->reference))
            ];

            if ($logsConfig === 'yes')
            {
                $log = new ThreeByFourOneyLogger();
                $log->debug('[PAYMENT API] PAYMENT REQUEST - CREDENTIALS ENVIRONMENT: ' . $environment);
                $log->debug('[PAYMENT API] PAYMENT REQUEST - ORDER REFERENCE: ' . $customerOrder->reference);
                $log->debug('[PAYMENT API] PAYMENT REQUEST - REQUEST BODY: ' . json_encode($requestBody));
            }

            $request = $this->payment_web_service->requestPayment($environment, $countryCode, $requestBody, $customerOrder->id_shop);
            
            if (is_object($request))
            {
                if (isset($request->returned_url)) {
                    $redirectionUrl = $request->returned_url;
                    if ($logsConfig === 'yes')
                    {
                        $log = new ThreeByFourOneyLogger();
                        $log->debug('[PAYMENT API] PAYMENT REQUEST CALLBACK - REDIRECT URL: ' . $redirectionUrl);
                    }
                    Tools::redirect($redirectionUrl);
                }
            } else {
                if ($logsConfig === 'yes')
                {
                    $log = new ThreeByFourOneyLogger();
                    $log->error('[PAYMENT API] PAYMENT REQUEST - NO REQUEST FOUND');
                }
                Tools::redirect($this->context->link->getPageLink('order', true));
            }
        } else {
            if ($logsConfig === 'yes')
            {
                $log = new ThreeByFourOneyLogger();
                $log->error('[PAYMENT API] PAYMENT REQUEST - NO BUSINESS TRANSACTION CODE RETURNED');
            }
            Tools::redirect($this->context->link->getPageLink('order', true));
        }
    }
}