<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyMarketingWebService.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneySimulationsModel.php';

require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
/**
 * ThreeByFourOneySimulationsModuleFrontController class
 * Front controller for simulations requests (mainly ajax manipulation)
 * 
 * @version 3.0.0
 */
class ThreeByFourOneySimulationsModuleFrontController extends ModuleFrontController
{
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return void
     */
    public function __construct()
    {
        $this->marketing_web_service = new ThreeByFourOneyMarketingWebService();
        $this->simulations = new ThreeByFourOneySimulationsModel();
        $this->utility = new ThreeByFourOneyUtility();
        parent::__construct();
    }

    /**
     * initContent function
     * Prestashop's init content function
     *
     * @version 3.0.0
     * @return void
     */
    public function initContent()
    {
        $this->ajax = true;
        parent::initContent();
    }

    /**
     * displayAjaxRequestSimulations function
     * Request simulations by requesting Marketing API or records if exists in module's simulations table.
     *
     * @version 3.0.0
     * @return void
     */
    public function displayAjaxRequestSimulations()
    {
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $paymentAmount = number_format((float)Tools::getValue('paymentAmount'), 2, '.', '');
        $countryCode = (string)Tools::getValue('customerCurrentCountryCode');
        $customerCurrentCountryCode = $this->utility->getConfigByLanguage('iso_code', strtolower($countryCode));
        $activeBusinessTransactionsCodes = Tools::getValue('activeBusinessTransactionsCodes');
        $activeBusinessTransactionsCodesArray = json_decode($activeBusinessTransactionsCodes);
        $shopId = (int)Context::getContext()->shop->id;
        $simulationsRecords = [];
        foreach ($activeBusinessTransactionsCodesArray as $key => $activeBusinessTransactionsCode)
        {
            $simulationsRecords[$key] = $this->simulations->getSimulations($activeBusinessTransactionsCode, $paymentAmount, $countryCode, $shopId);
            if ($simulationsRecords[$key] == null || empty($simulationsRecords[$key]))
            {
                $simulationsWebService = $this->marketing_web_service->requestSimulations($environmentConfig, $paymentAmount, $activeBusinessTransactionsCode, $countryCode);
                if ($simulationsWebService != null || !empty($simulationsWebService))
                {
                    $data = [];
                    // Simulation data
                    $data['business_transaction_code'] = (string)$simulationsWebService->business_transaction_details->business_transaction_code;
                    $data['payment_amount'] = (float)$simulationsWebService->payment_amount;
                    $data['effective_annual_percentage_rate'] = (float)$simulationsWebService->effective_annual_percentage_rate;
                    $data['administration_fee_amount'] = (float)$simulationsWebService->administration_fee_amount;
                    $data['total_cost'] = (float)$simulationsWebService->total_cost;
                    $data['down_payment_amount'] = (float)$simulationsWebService->down_payment_amount;
                    $data['nominal_annual_percentage_rate'] = (float)$simulationsWebService->nominal_annual_percentage_rate;
                    $data['first_instalment_date'] = ($customerCurrentCountryCode == 'FR') ? date("Y-m-d", strtotime((string)$simulationsWebService->first_instalment_date)) : (string)$simulationsWebService->first_instalment_date;
                    // Instalments management
                    if (isset($simulationsWebService->instalments))
                    {
                        $instalmentsLength = count($simulationsWebService->instalments);
                        $i = 0;
                        foreach ($simulationsWebService->instalments as $instalment)
                        {
                            if (++$i === $instalmentsLength)
                            {
                                $data['last_instalment_amount'] = (float)$instalment->instalment_amount;
                                $data['last_insurance_amount'] = (float)$instalment->insurance_amount;
                                $data['last_interest_amount'] = (float)$instalment->interest_amount;
                                $franceOverseasTerritoriesCodes = ['GP', 'GF', 'RE', 'MQ', 'YT', 'NC', 'PF', 'BL', 'MF', 'PM', 'WF'];
                                if (($countryCode == 'FR' || in_array($countryCode, $franceOverseasTerritoriesCodes)) && $instalment->instalment_number > 1)
                                {
                                    $data['last_instalment_number'] = (int)$instalment->instalment_number + 1;
                                } else {
                                    $data['last_instalment_number'] = (int)$instalment->instalment_number;
                                }
                                $data['last_collection_date'] = ($activeBusinessTransactionsCode == 'FR') ? date("Y-m-d", strtotime((string)$instalment->collection_date)) : (string)$instalment->collection_date;
                            }
                        }
                    }
                    // Context data
                    $data['country_code'] = (string)$countryCode;
                    $data['shop_id'] = (int)$shopId;
                    $this->simulations->insertSimulations($data);
                    $simulationsRecords[$key] = $this->simulations->getSimulations($data['business_transaction_code'], $data['payment_amount'], $data['country_code'], $shopId);
                }
            }
        }
        $this->context->smarty->assign(['simulations' => $simulationsRecords]);
        $output = $this->context->smarty->fetch(_PS_MODULE_DIR_ . 'threebyfouroney/views/templates/hook/simulation_tooltip/tooltip_content.tpl');
        $this->ajaxDie(json_encode($output));
    }
}