<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCredentialsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCarriersModel.php';
/**
 * ThreeByFourOneyMarketingUtility class
 * Module's utility class for several uses.
 *
 * @version 3.0.0
 */
class ThreeByFourOneyUtility
{
    protected $credentials;
    protected $carriers;
    /**
     * __construct function
     * Define variables for further manipulations.
     *
     * @version 3.0.0
     * @return mixed
     */
    public function __construct()
    {
        $this->credentials = new ThreeByFourOneyCredentialsModel();
        $this->carriers = new ThreeByFourOneyCarriersModel();
    }
    
    /**
     * checkPhpVersion function
     * Check PHP's version for prerequisites step
     *
     * @version 3.0.0
     * @return array
     */
    public function checkPhpVersion()
    {
        $phpVersionArray = [];

        if (phpversion() < '5.6')
        {
            $phpVersionArray = [
                'class'                     => 'danger',
                'minmum_required_version'   => '5.6',
                'php_version'               => phpversion(),
                'is_valid'                  => 'false'
            ];
        } else {
            $phpVersionArray = [
                'class'         => 'success',
                'php_version'   => phpversion(),
                'is_valid'      => 'true'
            ];
        }
        
        return $phpVersionArray;
    }

    /**
     * checkPrestashopVersion function
     * Check Prestashop's version for prerequisites step
     *
     * @version 3.0.0
     * @return array
     */
    public function checkPrestashopVersion()
    {
        $prestashopVersionArray = [];

        if (_PS_VERSION_ < '1.6.1')
        {
            $prestashopVersionArray = [
                'class'         => 'danger',
                'ps_version'    => _PS_VERSION_,
                'is_valid'      => 'false'
            ];
        } else {
            $prestashopVersionArray = [
                'class'         => 'success',
                'ps_version'    => _PS_VERSION_,
                'is_valid'      => 'true'
            ];
        }

        return $prestashopVersionArray;
    }
    /**
     * checkCurlExtension function
     * Check cURL extension presence for prerequisites step
     *
     * @version 3.0.0
     * @return array
     */
    public function checkCurlExtension()
    {
        $curlExtensioArray = [];
        $isCurlInstalled = function_exists('curl_version');

        if ($isCurlInstalled)
        {
            $curlExtensioArray = [
                'name'          => 'cURL',
                'class'         => 'success',
                'is_valid'      => 'true'
            ];
        } else {
            $curlExtensioArray = [
                'name'          => 'cURL',
                'class'         => 'danger',
                'is_valid'      => 'false'
            ];
        }

        return $curlExtensioArray;
    }

    /**
     * orderStatusesConfiguration function
     * Return the array of statuses and translations
     *
     * @version 3.0.2
     * @return array
     */
    private function orderStatusesConfiguration()
    {
        $orderStatusesArrary = [
            'TBFONEY_OS_PENDING' => [
                'default' => "Pending payment 3x4x Oney",
                'nl' => "In afwachting van betaling 3x4x Oney",
                'fr' => "En attente de paiement 3x4x Oney",
                'it' => "In attesa di pagamento 3x4x Oney",
                'pt' => "Pagamento pendente 3x4x Oney",
                'ro' => "Plată în așteptare 3x4x Oney",
                'es' => "Pago pendiente 3x4x Oney"

            ],
            'TBFONEY_OS_FAVORABLE' => [
                'default' => "Favorable opinion 3x4x Oney",
                'nl' => "Gunstig advies 3x4x Oney",
                'fr' => "Avis favorable 3x4x Oney",
                'it' => "Parere favorevole 3x4x Oney",
                'pt' => "Opinião favorável 3x4x Oney",
                'ro' => "Opinie favorabilă 3x4x Oney",
                'es' => "Opinión favorable 3x4x Oney"
            ],
            'TBFONEY_OS_READYTODELIVER' => [
                'default' => "Ready to deliver 3x4x Oney",
                'nl' => "Klaar om te leveren 3x4x Oney",
                'fr' => "Prêt à livrer 3x4x Oney",
                'it' => "Pronto a consegnare 3x4x Oney",
                'pt' => "Pronto para entregar Oney 3x4x",
                'ro' => "Gata de livrare 3x4x Oney",
                'es' => "Listo para entregar 3x4x Oney"
            ]
        ];
        return $orderStatusesArrary;
    }

    /**
     * orderStatusesConfiguration function
     * Handle Oney's module's custom statuses creation.
     *
     * @version 3.0.0|3.0.2
     * @return void
     */
    public function installOrderStatuses()
    {
        if (Configuration::get('PS_LANG_DEFAULT') == false) Configuration::updateValue('PS_LANG_DEFAULT', Context::getContext()->language->id);
        $orderStatusesConfig = $this->orderStatusesConfiguration();
        foreach ($orderStatusesConfig as $key => $os)
        {
            $statusName = [];
            $languages = Language::getLanguages();
            foreach ($languages as &$language)
            {
                $languageId = Language::getIdByIso($language['iso_code']);
                $countryCode = strtolower($language['iso_code']);
                if (isset($os[$countryCode]))
                {
                    $statusName[$languageId] = $os[$countryCode];
                } else {
                    $statusName[$languageId] = $os['default'];
                }
            }
            $orderStatus = new OrderState((int)Configuration::get($key));
            if ($orderStatus->module_name == 'threebyfouroney')
            {
                $orderStatus->deleted = 0;
                $orderStatus->name = $statusName;
                if (!$orderStatus->save()) return false;
            } else {
                $orderStatus = new OrderState();
                $orderStatus->module_name = 'threebyfouroney';
                $orderStatus->color = '#81bc00';
                $orderStatus->name = $statusName;
                if (!$orderStatus->add()) return false; else Configuration::updateValue($key, $orderStatus->id);
            }
            // Copy the object's icon
            $moduleOrderIcon = _PS_MODULE_DIR_ . 'threebyfouroney/views/img/order-icon.gif';
            $psOrderIcon = _PS_ROOT_DIR_ . '/img/os/' . (int)$orderStatus->id . '.gif';
            @copy($moduleOrderIcon, $psOrderIcon);
        }
        return true;
    }

    /**
     * installTabs function
     * Install admin tabs for Prestashop 1.6 compatibilities
     *
     * @version 3.0.0
     * @return void
     */
    public function installTabs()
    {
        $tabAdminThreeByFourOney = new Tab();
		$tabAdminThreeByFourOney->active = 1;
		$tabAdminThreeByFourOney->class_name = 'AdminThreeByFourOney';
		$tabAdminThreeByFourOney->name = array();
		foreach (Language::getLanguages(true) as &$lang)
			$tabAdminThreeByFourOney->name[$lang['id_lang']] = 'Threebyfouroney';
		$tabAdminThreeByFourOney->id_parent = -1;
		$tabAdminThreeByFourOney->module = 'threebyfouroney';
        $tabAdminThreeByFourOney->add();

        $tabAdminThreeByFourOneyConfirmation = new Tab();
		$tabAdminThreeByFourOneyConfirmation->active = 1;
		$tabAdminThreeByFourOneyConfirmation->class_name = 'AdminThreeByFourOneyConfirmation';
		$tabAdminThreeByFourOneyConfirmation->name = array();
		foreach (Language::getLanguages(true) as &$lang)
			$tabAdminThreeByFourOneyConfirmation->name[$lang['id_lang']] = 'Threebyfouroney';
		$tabAdminThreeByFourOneyConfirmation->id_parent = -1;
		$tabAdminThreeByFourOneyConfirmation->module = 'threebyfouroney';
        $tabAdminThreeByFourOneyConfirmation->add();

        $tabAdminThreeByFourOneyCancellation = new Tab();
		$tabAdminThreeByFourOneyCancellation->active = 1;
		$tabAdminThreeByFourOneyCancellation->class_name = 'AdminThreeByFourOneyCancellation';
		$tabAdminThreeByFourOneyCancellation->name = array();
		foreach (Language::getLanguages(true) as &$lang)
			$tabAdminThreeByFourOneyCancellation->name[$lang['id_lang']] = 'Threebyfouroney';
		$tabAdminThreeByFourOneyCancellation->id_parent = -1;
		$tabAdminThreeByFourOneyCancellation->module = 'threebyfouroney';
        $tabAdminThreeByFourOneyCancellation->add();

        $tabAdminThreeByFourOneyStatus = new Tab();
		$tabAdminThreeByFourOneyStatus->active = 1;
		$tabAdminThreeByFourOneyStatus->class_name = 'AdminThreeByFourOneyStatus';
		$tabAdminThreeByFourOneyStatus->name = array();
		foreach (Language::getLanguages(true) as &$lang)
			$tabAdminThreeByFourOneyStatus->name[$lang['id_lang']] = 'Threebyfouroney';
		$tabAdminThreeByFourOneyStatus->id_parent = -1;
		$tabAdminThreeByFourOneyStatus->module = 'threebyfouroney';
        $tabAdminThreeByFourOneyStatus->add();

        $tabAdminThreeByFourOneyOrdersUpdate = new Tab();
		$tabAdminThreeByFourOneyOrdersUpdate->active = 1;
		$tabAdminThreeByFourOneyOrdersUpdate->class_name = 'AdminThreeByFourOneyOrdersUpdate';
		$tabAdminThreeByFourOneyOrdersUpdate->name = array();
		foreach (Language::getLanguages(true) as &$lang)
			$tabAdminThreeByFourOneyOrdersUpdate->name[$lang['id_lang']] = 'Threebyfouroney';
		$tabAdminThreeByFourOneyOrdersUpdate->id_parent = -1;
		$tabAdminThreeByFourOneyOrdersUpdate->module = 'threebyfouroney';
        $tabAdminThreeByFourOneyOrdersUpdate->add();
    }

    /**
     * uninstallTabs function
     * Uninstall all admin tabs on module's uninstallation
     * 
     * @version 3.0.0
     * @return void
     */
    public function uninstallTabs()
    {
        $tabAdminThreeByFourOney = new Tab((int)Tab::getIdFromClassName('AdminThreeByFourOney'));
        $tabAdminThreeByFourOney->delete();
        $tabAdminThreeByFourOneyConfirmation = new Tab((int)Tab::getIdFromClassName('AdminThreeByFourOneyConfirmation'));
        $tabAdminThreeByFourOneyConfirmation->delete();
        $tabAdminThreeByFourOneyCancellation = new Tab((int)Tab::getIdFromClassName('AdminThreeByFourOneyCancellation'));
        $tabAdminThreeByFourOneyCancellation->delete();
        $tabAdminThreeByFourOneyStatus = new Tab((int)Tab::getIdFromClassName('AdminThreeByFourOneyStatus'));
        $tabAdminThreeByFourOneyStatus->delete();
        $tabAdminThreeByFourOneyOrdersUpdate = new Tab((int)Tab::getIdFromClassName('AdminThreeByFourOneyOrdersUpdate'));
        $tabAdminThreeByFourOneyOrdersUpdate->delete();
    }

    /**
     * installDefaultConfig function
     * Apply default configuration values to ps_configuration table depending on the shop
     *
     * @version 3.1.0
     * @param int $shopId
     * @return void
     */
    public function installConfiguration($shopId = 0)
    {
        if ($shopId != 0)
        {
            // Wizard values
            (bool)Configuration::updateValue('TBFONEY_IS_TERMS_ACCEPTED', 0, null, 0, $shopId);
            (int)Configuration::updateValue('TBFONEY_WIZARD_STEP', 1, null, 0, $shopId);
            (bool)Configuration::updateValue('TBFONEY_CARRIERS_CONFIGURED', 0, null, 0, $shopId);
            // Credentials configuration values
            (string)Configuration::updateValue('TBFONEY_ENVIRONMENT', 'staging', null, 0, $shopId);
            // Store and orders configuration values
            (string)Configuration::updateValue('TBFONEY_AUTOMATIC_CONFIRMATION', 'no', null, 0, $shopId);
            (int)Configuration::updateValue('TBFONEY_STORE_CATEGORY', 1, null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_HORIZONTAL_BANNER', 'show', null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_VERTICAL_BANNER', 'show', null, 0, $shopId);
            (bool)Configuration::updateValue('TBFONEY_SPECIFIC_COUNTRY_CONFIGURED', 0, null, 0, $shopId);
            // Advanced configuration values
            (string)Configuration::updateValue('TBFONEY_ACTIVATE_LOGS', 'yes', null, 0, $shopId);
            (int)Configuration::updateValue('TBFONEY_LOGS_LIFETIME', 10, null, 0, $shopId);
        }
    }

    public function installSpecificCountryConfiguration($credentialCountryCode, $shopId = 0)
    {
        $specificCountryConfig = (bool)Configuration::get('TBFONEY_SPECIFIC_COUNTRY_CONFIGURED');
        if (($credentialCountryCode === 'PT') && (!$specificCountryConfig))
        {
            // Specific country configuration values
            (bool)Configuration::updateValue('TBFONEY_SPECIFIC_COUNTRY_CONFIGURED', 1, null, 0, $shopId);
            (bool)Configuration::updateValue('TBFONEY_PT_IS_CONFIG_VALIDATED', 0, null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_PT_IS_CREDIT_INTERMEDIARY', 'yes', null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_PT_IS_EXCLUSIVITY_REGIME', 'no', null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION', null, null, 0, $shopId);
            (string)Configuration::updateValue('TBFONEY_PT_LEGAL_DOCUMENT', null, null, 0, $shopId);
        }
    }

    /**
     * uninstallConfiguration function
     * Uninstall all configuration on module's uninstallation
     * 
     * @version 3.1.0
     * @return void
     */
    public function uninstallConfiguration()
    {
        Configuration::deleteByName('TBFONEY_IS_TERMS_ACCEPTED');
        Configuration::deleteByName('TBFONEY_WIZARD_STEP');
        Configuration::deleteByName('TBFONEY_CARRIERS_CONFIGURED');
        Configuration::deleteByName('TBFONEY_ENVIRONMENT');
        Configuration::deleteByName('TBFONEY_AUTOMATIC_CONFIRMATION');
        Configuration::deleteByName('TBFONEY_STORE_CATEGORY');
        Configuration::deleteByName('TBFONEY_HORIZONTAL_BANNER');
        Configuration::deleteByName('TBFONEY_VERTICAL_BANNER');
        Configuration::deleteByName('TBFONEY_SPECIFIC_COUNTRY_CONFIGURED');
        Configuration::deleteByName('TBFONEY_PT_IS_CONFIG_VALIDATED');
        Configuration::deleteByName('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        Configuration::deleteByName('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        Configuration::deleteByName('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        Configuration::deleteByName('TBFONEY_PT_LEGAL_DOCUMENT');
        Configuration::deleteByName('TBFONEY_ACTIVATE_LOGS');
        Configuration::deleteByName('TBFONEY_LOGS_LIFETIME');
    }
    
    /**
     * checkCredentialsValidity function
     * Verify validity of recorded credentials and update is_valid column in module's credentials table
     *
     * @version 3.0.0
     * @param string $environment
     * @return void
     */
    public function checkCredentialsValidity($environment)
    {
        $shopId = (int)Context::getContext()->shop->id;
        $validityRecords = $this->credentials->getCredentialsValidity($environment, $shopId);
        if ($validityRecords != null)
        {
            foreach ($validityRecords as &$record) {
                return (bool)$record['is_valid'] ? 'true' : 'false';
            }
        } else {
            return 'false';
        }
    }

    /**
     * credentialsManament function
     * Return merchant's credentials based on selected environment and customer's contry code.
     *
     * @version 3.0.0
     * @return array
     */
    public function credentialsManagement()
    {
        $environment = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $shopId = (int)Context::getContext()->shop->id;
        $customerCurrentCountryCode = $this->customerCountryConfiguration();
        $credentials = $this->credentials->getCredentials($environment, $customerCurrentCountryCode, $shopId);
        foreach ($credentials as $credential)
        {
            $data = [
                'environment'       => $credential['environment'],
                'merchant_guid'     => $credential['merchant_guid'],
                'psp_guid'          => $credential['psp_guid']
            ];
            return $data;
        }
    }

    /**
     * customerCountryConfiguration function
     * Define customer's country code based on geolcation, billing address or language selection
     *
     * @version 3.0.0
     * @return string
     */
    public function customerCountryConfiguration()
    {
        $customerCodeCountry = 0;
        $record = null;
        if (_PS_VERSION_ <= '1.7')
        {
            if (class_exists('GeoIp2\Database\Reader') && file_exists(_PS_GEOIP_DIR_ . _PS_GEOIP_CITY_FILE_))
            {
                $reader = new GeoIp2\Database\Reader(_PS_GEOIP_DIR_ . _PS_GEOIP_CITY_FILE_);
                $record = $reader->city(Tools::getRemoteAddr());
                if (is_object($record))
                {
                    $customerCodeCountry = $record->raw['country']['iso_code'];
                }
            } elseif (file_exists(_PS_GEOIP_DIR_ . 'GeoLiteCity.dat'))
            {
                include_once _PS_GEOIP_DIR_ . 'geoipcity.inc';
                $geoIpCity = geoip_open(realpath(_PS_GEOIP_DIR_ . 'GeoLiteCity.dat'), GEOIP_STANDARD);
                $record = geoip_record_by_addr($geoIpCity, Tools::getRemoteAddr());
                if (is_object($record))
                {
                    $customerCodeCountry = $record->country_code;
                }
            }
        }

        if (isset(Context::getContext()->cart->id_address_invoice))
        {
            if (($invoiceId = Context::getContext()->cart->id_address_invoice) != 0)
            {
                $invoiceAddress = Address::getCountryAndState($invoiceId);
                $customerCodeCountry = Country::getIsoById($invoiceAddress['id_country']);
            } elseif ($record == null) {
                $customerCodeCountry = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));
            }
        } elseif (isset(Context::getContext()->cookie->id_cart)) {
            $cart = new Cart(Context::getContext()->cookie->id_cart);
            if (($invoiceId = $cart->id_address_invoice) != 0)
            {
                $invoiceAddress = Address::getCountryAndState($invoiceId);
                $customerCodeCountry = Country::getIsoById($invoiceAddress['id_country']);
            } elseif ($record == null) {
                $customerCodeCountry = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));
            }
        } elseif (isset(Context::getContext()->language->iso_code)) {
            $customerCodeCountry = Context::getContext()->language->iso_code;
        } elseif ($record == null) {
            $customerCodeCountry = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));
        }
        return $customerCodeCountry;
    }
    /**
     * scopeCountriesConfiguration function
     * Define Oney's opened countries configuration for further manipulations.
     *
     * @version 3.0.0
     * @return mixed
     */
    public function scopeCountriesConfiguration()
    {
        $scopeCountriesArray = [
            'be' => [
                'iso_code'          => 'BE',
                'name'              => 'Belgium',
                'country_reference' => 'BE',
                'iso_alpha3'        => 'BEL',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+32)|(^0032)|(^0))4([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{4}$/'
            ],
            'it' => [
                'iso_code'          => "IT",
                'name'              => "Italy",
                'country_reference' => "IT",
                'iso_alpha3'        => "ITA",
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+39)|(^0039)|(^\0{0}))3([0-9]){9}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'fr' => [
                'iso_code'          => 'FR',
                'name'              => 'France',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+33)|(^0033)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'gp' => [
                'iso_code'          => 'FR',
                'name'              => 'Guadeloupe',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+590)|(^00590)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'gf' => [
                'iso_code'          => 'FR',
                'name'              => 'Guyane Française',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+594)|(^00594)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            're' => [
                'iso_code'          => 'FR',
                'name'              => 'Réunion',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+262)|(^00262)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'mq' => [
                'iso_code'          => 'FR',
                'name'              => 'Martinique',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+596)|(^00596)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'yt' => [
                'iso_code'          => 'FR',
                'name'              => 'Mayotte',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+262)|(^00262)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'nc' => [
                'iso_code'          => 'FR',
                'name'              => 'Nouvelle-Calédonie',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+687)|(^00687)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            // 'pf' => [
            //     'iso_code'          => 'FR',
            //     'name'              => 'Polynésie Française',
            //     'country_reference' => 'FR',
            //     'iso_alpha3'        => 'FRA',
            //     'currency_code'     => '',
            //     'phone_regex'       => '/((^\+689)|(^00689)|(^0))[6-7]([0-9]){7,8}$/',
            //     'zip_code_regex'    => '/^\d{5}$/'
            // ],
            'bl' => [
                'iso_code'          => 'FR',
                'name'              => 'Saint-Barthélémy',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+590)|(^00590)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'mf' => [
                'iso_code'          => 'FR',
                'name'              => 'Saint-Martin',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+590)|(^00590)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'pm' => [
                'iso_code'          => 'FR',
                'name'              => 'Saint-Pierre-et-Miquelon',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+508)|(^00508)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            // 'wf' => [
            //     'iso_code'          => 'FR',
            //     'name'              => 'Wallis-et-Futuna',
            //     'country_reference' => 'FR',
            //     'iso_alpha3'        => 'FRA',
            //     'currency_code'     => '',
            //     'phone_regex'       => '/((^\+681)|(^00681)|(^0))[6-7]([0-9]){7,8}$/',
            //     'zip_code_regex'    => '/^\d{5}$/'
            // ],
            'pt' => [
                'iso_code'          => 'PT',
                'name'              => 'Portugal',
                'country_reference' => 'PT',
                'iso_alpha3'        => 'PRT',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+351)|(^00351)|(^\0{0}))9[1,2,3,6]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{4}[- ]{0,1}\d{3}$/'
            ],
            'es' => [
                'iso_code'          => 'SP',
                'name'              => 'Spain',
                'country_reference' => 'ES',
                'iso_alpha3'        => 'ESP',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+34)|(^0034)|(^\0{0}))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ],
            'ro' => [
                'iso_code'          => 'RO',
                'name'              => 'Romania',
                'country_reference' => 'RO',
                'iso_alpha3'        => 'ROU',
                'currency_code'     => 'RON',
                'phone_regex'       => '/((^\+40)|(^0040)|(^0))7([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{6}$/'
            ],
            'default' => [
                'iso_code'          => 'FR',
                'name'              => 'France',
                'country_reference' => 'FR',
                'iso_alpha3'        => 'FRA',
                'currency_code'     => 'EUR',
                'phone_regex'       => '/((^\+33)|(^0033)|(^0))[6-7]([0-9]){7,8}$/',
                'zip_code_regex'    => '/^\d{5}$/'
            ]
        ];

        return $scopeCountriesArray;
    }

    /**
     * countriesMapper function
     * Match Prestashop's countries config with Oney's scope countries.
     *
     * @version 3.0.0
     * @return array
     */
    public function countriesMapper()
    {
        $psCountries = [];
        if (isset(Context::getContext()->employee->id))
        {
            $psCountries = Country::getCountries((int)Context::getContext()->employee->id_lang, true);
        } else {
            $psCountries = Country::getCountries((int)Context::getContext()->customer->id_lang, true);
        }
        $scopeCountries = $this->scopeCountriesConfiguration();
        $matchedCountries = [];
        foreach ($psCountries as $psCountry)
        {
            foreach (array_keys($scopeCountries) as $key)
            {
                if (strtoupper($psCountry['iso_code']) == strtoupper($key))
                {
                    $matchedCountries[] = $psCountry;
                }
            }
        }
        
        return $matchedCountries;
    }

    /**
     * getConfigByLanguage function
     * Retrun attribute value from Oney's opened countries configuration based on country code passed in argument.
     *
     * @version 3.0.0
     * @param string $key
     * @param string $countryCode
     * @return mixed
     */
    public function getConfigByLanguage($key, $countryCode)
    {
        $scopeCountriesConfiguration = $this->scopeCountriesConfiguration();
        if (isset($scopeCountriesConfiguration[$countryCode]))
        {
            if (isset($scopeCountriesConfiguration[$countryCode][$key]))
            {
                return $scopeCountriesConfiguration[$countryCode][$key];
            }
        } else {
            if (isset($scopeCountriesConfiguration['default'][$key]))
            {
                return $scopeCountriesConfiguration['default'][$key];
            }
        }
        return [];
    }

    /**
     * currencyDisplayCondition function
     * Define display condition based on currency configuration
     * If the selected currency by the customer is listed in the module's configuration then display element
     *
     * @version 3.0.4
     * @return bool
     */
    public function currencyDisplayCondition()
    {
        $customerCurrentCurrency = Context::getContext()->currency->iso_code;
        $countriesConfig = $this->scopeCountriesConfiguration();
        $currenciesConfig = array_column($countriesConfig, 'currency_code');
        $searchCurrenciesConfig = in_array($customerCurrentCurrency, $currenciesConfig);
        return $searchCurrenciesConfig;
    }

    /**
     * splitAddresses function
     * Split end customer's address mainly for request body manipulation. Split after 38 characters to lineX attribute.
     *
     * @param string $address
     * @param integer $maxLength
     * @return array
     */
    public function splitAddresses($address, $maxLength = 38)
    {
        $array = [];
        if (strlen($address) > $maxLength)
        {
            $array[] = substr($address, 0, strrpos(substr($address, 0, $maxLength), ' '));
            $array[] = substr($address, strrpos(substr($address, 0, $maxLength), ' '));

            foreach ($array as $key => $value) {
                if (strlen($value) > $maxLength) {
                    $array[$key] = substr($value, 0, (strrpos(substr($value, 0, $maxLength), ' ') - 1));
                    $array[] = substr($value, strrpos(substr($value, 0, $maxLength), ' ') - 1);
                }
            }
        } else {
            $array[] = $address;
        }

        return array_map('trim', $array);
    }

    /**
     * getLegalFile function
     * Get Portugal legal file path.
     *
     * @version 3.0.0|3.0.3
     * @return bool|string
     */
    public static function getLegalFile()
    {
        $path = Configuration::get('TBFONEY_PT_LEGAL_DOCUMENT');
        if (!empty($path)) return $path;
        return false;
    }

    /**
     * deleteLegalFile function
     * Upload Portugal legal document function.
     *
     * @version 3.0.0|3.0.3
     * @param string $destinationDirectory
     * @param string $currentLegalFile
     * @return void
     */
    public function deleteLegalFile($destinationDirectory, $currentLegalFile)
    {
        $files = scandir($destinationDirectory);
        foreach ($files as &$file)
        {
            if ($file == $currentLegalFile || $file == "index.php" || $file == "." || $file == "..") continue;
            unlink($destinationDirectory . $file);
        }
    }

    /**
     * getBusinessTransactionType function
     * Get Business transaction type based on its code.
     *
     * @version 3.0.5
     * @param string $btCode
     * @return string
     */
    public function getBusinessTransactionType($btCode)
    {
        if (preg_match("/^[0-9]/", $btCode)) {
            return "SP";
        } elseif (preg_match("/^PL/", $btCode)) {
            return "PL";
        }
    }

    /**
     * stockAvailableManagement function
     * Update stock quantities on non final order status
     *
     * @version 3.0.0
     * @param object $order
     * @param int $shopId
     * @return array
     */
    public function stockAvailableManagement($order, $shopId)
    {
        foreach ($order->getCartProducts() as $product)
        {
            StockAvailableCore::updateQuantity($product['id_product'], $product['id_product_attribute'], $product['product_quantity'], $shopId, false, []);
        }
    }
}
