<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

/**
 * ThreeByFourOneyBusinessTransactionsModel class
 * Create specific table for business transactions records.
 *
 * @version 3.0.0
*/
class ThreeByFourOneyBusinessTransactionsModel extends ObjectModelCore
{
    public $id;
    public $business_transaction_code;
    public $business_transaction_version;
    public $business_transaction_type;
    public $customer_label;
    public $short_label;
    public $long_label;
    public $minimum_number_of_instalments;
    public $maximum_number_of_instalments;
    public $minimum_selling_price;
    public $maximum_selling_price;
    public $discount_rate;
    public $creation_datetime;
    public $choosable_number_of_instalments;
    public $free_business_transaction;
    public $is_active;
    public $country_code;
    public $shop_id;

    public static $definition = array(
        'table' => 'threebyfouroney_business_transactions',
        'primary' => 'id',
        'multilang' => false,
        'fields' => array(
            'business_transaction_code'         => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'business_transaction_version'      => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'business_transaction_type'         => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'customer_label'                    => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'short_label'                       => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'long_label'                        => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'minimum_number_of_instalments'     => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'maximum_number_of_instalments'     => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'minimum_selling_price'             => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'maximum_selling_price'             => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'discount_rate'                     => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'choosable_number_of_instalments'   => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'free_business_transaction'         => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'is_active'                         => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
            'country_code'                      => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'shop_id'                           => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
        )
    );

    /**
     * getBusinessTransactions function
     * Get all business transactions recorded in threebyfouroney_business_transactions table
     *
     * @version 3.0.0|3.0.5
     * @param string $countryCode
     * @param int $shopId
     * @return void
     */
    public function getBusinessTransactions($countryCode, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'bt');
        $sql->where('bt.country_code = "' . $countryCode . '"');
        $sql->where('bt.shop_id = "' . $shopId . '"');
        $sql->orderBy('minimum_number_of_instalments');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getActiveBTInstalments function
     * Get all instalements and charges from recorded active business trancations in threebyfouroneymarketing_business_transactions table
     *
     * @version 3.0.0|3.0.5
     * @param string $countryCode
     */
    public function getActiveBTInstalments($countryCode, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('minimum_number_of_instalments, min(minimum_selling_price) as minimum_selling_price, max(maximum_selling_price) as maximum_selling_price, business_transaction_code, free_business_transaction');
        $sql->from(self::$definition['table'], 'bt');
        $sql->where('bt.country_code = "' . $countryCode . '"');
        $sql->where('bt.is_active = true');
        $sql->where('bt.shop_id = "' . $shopId . '"');
        $sql->orderBy('minimum_number_of_instalments');
        $sql->groupBy('minimum_number_of_instalments');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getActiveBTInstalmentWithFilter function
     * Based on parameters, get all instalments and prices from recorded active business transactions in threebyfouroneymarketing_business_transactions table
     *
     * @version 3.0.0|3.0.5
     * @param string $price
     * @param string $countryCode
     * @return array
     */
    public function getActiveBTInstalmentWithFilter($price, $countryCode, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('business_transaction_code, short_label, customer_label, minimum_number_of_instalments, minimum_selling_price, maximum_selling_price, free_business_transaction');
        $sql->from(self::$definition['table'], 'bt');
        $sql->where('bt.minimum_selling_price <= "' . $price . '"');
        $sql->where('bt.maximum_selling_price >= "' . $price . '"');
        $sql->where('bt.is_active = true');
        $sql->where('bt.country_code = "' . $countryCode . '"');
        $sql->where('bt.shop_id = "' . $shopId . '"');
        $sql->orderBy('minimum_number_of_instalments');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * insertBusinessTransactions function
     * Insert business trancations with datas sent in parameter
     *
     * @version 3.0.0
     * @param array $data
     * @return void
     */
    public function insertBusinessTransactions($data)
    {
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    /**
     * updateBusinessTransactions function
     * Update business trancations with datas sent in parameter
     *
     * @version 3.0.0|3.0.5
     * @param array $data
     * @param string $countryCode
     * @return void
     */
    public function updateBusinessTransactions($data, $countryCode, $shopId)
    {
        return Db::getInstance()->update(
            self::$definition['table'],
            [
                'is_active' => $data['is_active']
            ],
            'business_transaction_code = "' . $data['business_transaction_code'] . '" && country_code = "' . $countryCode . '" && shop_id = "' . $shopId . '"'
        );
    }

    /**
     * deleteBusinessTransactions function
     * Delete all business transactions
     *
     * @version 3.0.0|3.0.5
     * @return void
     */
    public function deleteBusinessTransactions($shopId)
    {
        return Db::getInstance()->delete(
            self::$definition['table'],
            'shop_id = "' . $shopId . '"'
        );
    }
}
