<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Marketing
 */

/**
 * ThreeByFourOneyCarriersModel class
 * Create specific table for carriers records.
 *
 * @version 3.0.0
*/
class ThreeByFourOneyCarriersModel extends ObjectModelCore
{
    public $id;
    public $address_type;
    public $delivery_mode;
    public $delivery_option;
    public $priority_delivery;
    public $store_id;
    public $carrier_id;
    public $shop_id;

    public static $definition = array(
        'table' => 'threebyfouroney_carriers',
        'primary' => 'id',
        'multilang' => false,
        'fields' => array(
            'address_type'         => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'delivery_mode'        => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'delivery_option'      => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'priority_delivery'    => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'store_id'             => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'carrier_id'           => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'shop_id'              => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true)
        )
    );

    /**
     * getCarriers function
     * Get all carriers recorded in threebyfouroney_carriers table
     *
     * @version 3.1.0
     * @param string $countryCode
     * @return array
     */
    public function getCarriers($shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'c');
        $sql->where('c.shop_id = "' . $shopId . '"');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getStores function
     * Get stores from ps_store table
     *
     * @version 3.1.11
     * @return array
     */
    public function getStores($langId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('store', 's');
        if (_PS_VERSION_ > '1.7') {
            $sql->innerJoin('store_lang', 'sl', 'sl.id_store = s.id_store');
            $sql->where('sl.id_lang = "' . $langId . '"');
        }
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getStoreById function
     * Get store for specific carrier recorded in threebyfouroney_carriers table
     *
     * @version 3.1.11
     * @param int $storeId
     * @return array
     */
    public function getStoreById($storeId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('store', 's');
        if (_PS_VERSION_ > '1.7') {
            $sql->where('s.id_store = "' . (int)$storeId . '"');
            $sql->innerJoin('store_lang', 'sl', 'sl.id_store = s.id_store');
        }
        return Db::getInstance()->getRow($sql);
    }

    /**
     * getCarriersWithParams function
     * Based on parameters, get all carriers recorded in threebyfouroney_carriers table
     *
     * @version 3.0.0|3.0.5
     * @param int $carrierId
     * @param int $shopId
     * @return object
     */
    public function getCarriersWithParams($carrierId, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'c');
        $sql->where('c.carrier_id = "' . $carrierId . '"');
        $sql->where('c.shop_id = "' . $shopId . '"');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * insertCarriers function
     * Insert carriers with datas sent in parameter
     *
     * @version 3.0.0|3.0.5
     * @param int $carrierId
     * @param array $data
     * @return void
     */
    public function insertCarriers($carrierId, $data)
    {
        $data = [
            'address_type'          => $data['address_type'],
            'delivery_mode'         => $data['delivery_mode'],
            'delivery_option'       => $data['delivery_option'],
            'priority_delivery'     => $data['priority_delivery'],
            'store_id'              => $data['store_id'],
            'carrier_id'            => $carrierId,
            'shop_id'               => $data['shop_id']
        ];
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    /**
     * updateCarriers function
     * Update carriers with datas sent in parameter
     *
     * @version 3.0.0|3.0.5
     * @param int $carrierId
     * @param array $data
     * @return void
     */
    public function updateCarriers($carrierId, $data)
    {
        $data = [
            'address_type'          => $data['address_type'],
            'delivery_mode'         => $data['delivery_mode'],
            'delivery_option'       => $data['delivery_option'],
            'priority_delivery'     => $data['priority_delivery'],
            'store_id'              => $data['store_id'],
            'carrier_id'            => $carrierId,
            'shop_id'               => $data['shop_id']
        ];
        return Db::getInstance()->update(
            self::$definition['table'],
            $data,
            'carrier_id = "' . $carrierId . '" && shop_id = "' . $data['shop_id'] . '"'
        );
    }
}
