<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

/**
 * ThreeByFourOneyCredentialsModel class
 * Create specific table for credentials records.
 *
 * @version 3.0.0
*/
class ThreeByFourOneyCredentialsModel extends ObjectModelCore
{
    public $id;
    public $environment;
    public $merchant_guid;
    public $marketing_api_key;
    public $country_code;
    public $is_valid;
    public $shop_id;

    public static $definition = array(
        'table' => 'threebyfouroney_credentials',
        'primary' => 'id',
        'multilang' => false,
        'fields' => array(
            'environment'               => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'merchant_guid'             => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'psp_guid'                  => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'marketing_api_key'         => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'payment_api_key'           => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'country_code'              => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'is_valid'                  => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'shop_id'                   => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true)
        )
    );

    /**
     * getCredentials function
     * Return credentials based on parameters
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @return object
     */
    public function getCredentials($environment, $countryCode, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'c');
        $sql->where('c.country_code = "' . (string)$countryCode . '"');
        $sql->where('c.environment = "' . (string)$environment . '"');
        $sql->where('c.shop_id = "' . (int)$shopId . '"');
        $sql->groupBy('c.country_code && c.environment');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getCredentialsValidity function
     * Declare validity over credentials
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @return void
     */
    public function getCredentialsValidity($environment, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('is_valid');
        $sql->from(self::$definition['table'], 'c');
        $sql->where('c.environment = "' . (string)$environment . '"');
        $sql->where('c.shop_id = "' . (int)$shopId . '"');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * insertCredentials function
     * Insert credentails based on parameters
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param array $data
     * @param string $countryCode
     * @return void
     */
    public function insertCredentials($environment, $countryCode, $data = [])
    {
        $data = [
            'environment'           => (string)$environment,
            'merchant_guid'         => (string)$data['merchant_guid'],
            'psp_guid'              => (string)$data['psp_guid'],
            'marketing_api_key'     => (string)$data['marketing_api_key'],
            'payment_api_key'       => (string)$data['payment_api_key'],
            'country_code'          => (string)$countryCode,
            'is_valid'              => (int)$data['is_valid'],
            'shop_id'               => (int)$data['shop_id']
        ];
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    /**
     * updateCredentials function
     * Update credentails based on validity
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param array $data
     * @param string $countryCode
     * @return bool
     */
    public function updateCredentials($environment, $countryCode, $data = [])
    {
        $data = [
            'environment'           => (string)$environment,
            'merchant_guid'         => (string)$data['merchant_guid'],
            'psp_guid'              => (string)$data['psp_guid'],
            'marketing_api_key'     => (string)$data['marketing_api_key'],
            'payment_api_key'       => (string)$data['payment_api_key'],
            'country_code'          => (string)$countryCode,
            'is_valid'              => (int)$data['is_valid'],
            'shop_id'               => (int)$data['shop_id']
        ];
        return Db::getInstance()->update(
            self::$definition['table'],
            $data,
            'country_code = "' . (string)$countryCode . '" && environment = "' . (string)$environment . '" && shop_id = "' . (int)$data['shop_id'] . '"'
        );
    }

    /**
     * updateCredentialsValidity function
     * Update credetials validity for any status changes
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param array $data
     * @param string $countryCode
     * @return void
     */
    public function updateCredentialsValidity($environment, $countryCode, $data = [])
    {
        $data = [
            'environment'           => (string)$environment,
            'merchant_guid'         => (string)$data['merchant_guid'],
            'psp_guid'              => (string)$data['psp_guid'],
            'marketing_api_key'     => (string)$data['marketing_api_key'],
            'payment_api_key'       => (string)$data['payment_api_key'],
            'country_code'          => (string)$countryCode,
            'is_valid'              => (int)$data['is_valid'],
            'shop_id'               => (int)$data['shop_id']
        ];
        return Db::getInstance()->update(
            self::$definition['table'],
            $data['is_valid'],
            'merchant_guid = "' . (string)$data['merchant_guid'] . '" && psp_guid = "' . (string)$data['psp_guid'] . '" && marketing_api_key = "' . (string)$data['marketing_api_key'] . '" && payment_api_key = "' . (string)$data['payment_api_key'] . '" && country_code = "' . (string)$countryCode . '" && environment = "' . (string)$environment . '"' . '" && shop_id = "' . (int)$data['shop_id'] . '"'
        );
    }

    /**
     * deleteCredentials function
     * Remove all credentials
     *
     * @version 3.0.0|3.0.5
     * @param string $environment
     * @param string $countryCode
     * @return void
     */
    public function deleteCredentials($environment, $countryCode, $shopId)
    {
        return Db::getInstance()->delete(
            self::$definition['table'],
            'country_code = "' . (string)$countryCode . '" && environment = "' . (string)$environment . '" && shop_id = "' . (int)$shopId . '"'
        );
    }
}
