<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Marketing
 */

/**
 * ThreeByFourOneyOrdersModel class
 * Create specific table for orders records.
 *
 * @version 3.0.0
*/
class ThreeByFourOneyOrdersModel extends ObjectModelCore
{
    public $id;
    public $id_order;
    public $reference;
    public $amount;
    public $country_code;

    public $initial_state;
    public $final_state;
    public $date_add;
    public $date_upd;

    public static $definition = array(
        'table' => 'threebyfouroney_orders',
        'primary' => 'id',
        'multilang' => false,
        'fields' => array(
            'id_order'      => array('type' => self::TYPE_INT, 'validate' => 'isString', 'required' => true),
            'reference'     => array('type' => self::TYPE_STRING, 'validate' => 'isInt', 'required' => true),
            'amount'        => array('type' => self::TYPE_FLOAT, 'validate' => 'isFloat', 'required' => true),
            'country_code'  => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'shop_id'       => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'initial_state' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'final_state'   => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'date_add'      => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_upd'      => array('type' => self::TYPE_DATE, 'validate' => 'isDate')
        )
    );

    /**
     * getOrderByReference function
     * Return order based on it's refenrece
     *
     * @version 3.0.4
     * @param string $reference
     * @param string $countryCode
     * @return object
     */
    public function getOrderByReference($reference, $countryCode, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'o');
        $sql->where('o.reference = "' . (string)$reference . '"');
        $sql->where('o.country_code = "' . (string)$countryCode . '"');
        $sql->where('o.shop_id = "' . (int)$shopId . '"');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * getOrderById function
     * Return order based on it's id
     *
     * @version 3.0.4
     * @param int $idOrder
     * @return object
     */
    public function getOrderById($idOrder, $shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'o');
        $sql->where('o.id_order = "' . (int)$idOrder . '"');
        $sql->where('o.shop_id = "' . (int)$shopId . '"');
        return Db::getInstance()->getRow($sql);
    }

    /**
     * getOrders function
     * Get all orders recorded in threebyfouroney_orders table
     *
     * @version 3.0.0
     * @return object
     */
    public function getOrdersByShopId($shopId)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 'o');
        $sql->where('o.shop_id = "' . (int)$shopId . '"');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * insertOrder function
     * Insert order with datas sent in parameter
     *
     * @version 3.0.0
     * @param array $data
     * @return void
     */
    public function insertOrder($data)
    {
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    /**
     * updateOrders function
     * Update orders with datas sent in parameter
     *
     * @version 3.0.0
     * @param int $orderId
     * @param array $data
     * @return void
     */
    public function updateOrder($orderId, $amount)
    {
        return Db::getInstance()->update(
            self::$definition['table'],
            [
                'amount' => $amount
            ],
            'id_order = ' . $orderId
        );
    }

    /**
     * initialize function
     * Update orders to be flagged as intialized
     *
     * @version 3.1.8
     * @param int $orderId
     * @return bool
     **/
    public function initialize($orderId) {
        return Db::getInstance()->update(
            self::$definition['table'],
            [
                'initial_state' => 0,
                'date_upd' => date('Y-m-d H:i:s')
            ],
            'id_order = ' . $orderId
        );
    }

    /**
    * finalize function
    * Update orders to be flagged as final state
    *
    * @version 3.1.8
    * @param int $orderId
    * @return bool
    **/
    public function finalize($orderId) {
        return Db::getInstance()->update(
          self::$definition['table'],
          [
              'final_state' => 1,
              'date_upd' => date('Y-m-d H:i:s')
          ],
          'id_order = ' . $orderId
        );
    }

    /**
    * checkState function
    * Check if an order has not been to a specific state
    *
    * @version 3.1.8
    * @param int $orderId
    * @param int $stateId
    * @return bool
    **/
    public function checkStatus($orderId, $stateId)  {
        return ((int)Db::getInstance()->getValue("SELECT COUNT(*) FROM " . _DB_PREFIX_ . "order_history WHERE id_order = $orderId AND id_order_state = $stateId") == 0);
    }

    /**
     * getStockReservedQuantity function
     * Get reserved quantity for a specific product attribute
     *
     * @version 3.0.0
     * @param int $productId
     * @param int $productAttributeId
     * @return array
     */
    public function getStockReservedQuantity($productId, $productAttributeId)
    {
        $sql = new DbQuery();
        (_PS_VERSION_ < '1.7.0') ? $sql->select('SUM(quantity)') : $sql->select('SUM(reserved_quantity)');
        $sql->from('stock_available', 'sa');
        $sql->where('sa.id_product = "' . (int)$productId . '"');
        $sql->where('sa.id_product_attribute = "' . (int)$productAttributeId . '"');
        return Db::getInstance()->getValue($sql);
    }
}
