<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

/**
 * ThreeByFourOneySimulationsModel class
 * Create specific table for simulations records.
 *
 * @version 3.0.0
*/
class ThreeByFourOneySimulationsModel extends ObjectModelCore
{
    public $id;
    public $business_transaction_code;
    public $payment_amount;
    public $effective_annual_percentage_rate;
    public $administration_fee_amount;
    public $total_cost;
    public $down_payment_amount;
    public $nominal_annual_percentage_rate;
    public $first_instalment_date;
    public $instalment_amount;
    public $insurance_amount;
    public $interest_amount;
    public $instalment_number;
    public $collection_date;
    public $country_code;
    public $shop_id;

    public static $definition = array(
        'table' => 'threebyfouroney_simulations',
        'primary' => 'id',
        'multilang' => false,
        'fields' => array(
            'business_transaction_code'             => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'payment_amount'                        => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'effective_annual_percentage_rate'      => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'administration_fee_amount'             => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'total_cost'                            => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'down_payment_amount'                   => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'nominal_annual_percentage_rate'        => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => true),
            'first_instalment_date'                 => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'last_instalment_amount'                => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'last_insurance_amount'                 => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'last_interest_amount'                  => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'last_instalment_number'                => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
            'last_collection_date'                  => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'country_code'                          => array('type' => self::TYPE_STRING, 'validate' => 'isString'),
            'shop_id'                               => array('type' => self::TYPE_INT, 'validate' => 'isInt'),
        )
    );

    /**
     * getSimulations function
     * Based on parameters, get all simulations recorded in threebyfouroneymarketing_simulations table
     *
     * @version 3.0.0
     * @param float $paymentAmount
     * @param string $businessTransactionCode
     * @param string $countryCode
     * @param int $shopId
     * @return array
     */
    public function getSimulations($businessTransactionCode, $paymentAmount, $countryCode, $shopId = false)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from(self::$definition['table'], 's');
        $sql->where('s.business_transaction_code = "' . $businessTransactionCode . '"');
        $sql->where('s.payment_amount = "' . $paymentAmount . '"');
        $sql->where('s.country_code = "' . $countryCode . '"');
        if($shopId) $sql->where('s.shop_id = ' . $shopId);
        $sql->orderBy('s.last_instalment_number ASC');
        return Db::getInstance()->executeS($sql);
    }

    /**
     * insertSimulations function
     * Insert simulations with datas sent in parameter
     *
     * @version 3.0.0
     * @param array $data
     * @return void
     */
    public function insertSimulations($data)
    {
        return Db::getInstance()->insert(self::$definition['table'], $data);
    }

    /**
     * deleteSimulations function
     * Delete all simulations
     *
     * @version 3.0.0
     * @return void
     */
    public function deleteSimulations()
    {
        return Db::getInstance()->delete(self::$definition['table']);
    }
}
