<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

$sql = [];

// Create Carriers table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneyCarriersModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `address_type` tinyint(1) NOT NULL,
    `delivery_mode` tinyint(1) NOT NULL,
    `delivery_option` tinyint(1) NOT NULL,
    `priority_delivery` tinyint(1) NOT NULL,
    `store_id` int(10) NOT NULL,
    `carrier_id` int(10) NOT NULL,
    `shop_id` int(10) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

// Create Credentials table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneyCredentialsModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `environment` varchar(15) NULL,
    `merchant_guid` varchar(255) NULL,
    `psp_guid` varchar(255) NULL,
    `marketing_api_key` varchar(255) NULL,
    `payment_api_key` varchar(255) NULL,
    `country_code` varchar(2) NULL,
    `is_valid` boolean,
    `shop_id` int(10) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

// Create business transactions table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneyBusinessTransactionsModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `business_transaction_code` varchar(10) NULL,
    `business_transaction_version` int(2) NULL,
    `business_transaction_type` varchar(255) NULL,
    `customer_label` varchar(255) NULL,
    `short_label` varchar(100) NULL,
    `long_label` varchar(100) NULL,
    `minimum_number_of_instalments` int(2) NOT NULL,
    `maximum_number_of_instalments` int(2) NOT NULL,
    `minimum_selling_price` decimal(15,2) NULL,
    `maximum_selling_price` decimal(15,2) NULL,
    `discount_rate` decimal(15,2) NULL,
    `choosable_number_of_instalments` boolean,
    `free_business_transaction` boolean,
    `is_active` boolean,
    `country_code` varchar(2) NULL,
    `shop_id` int(10) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

// Create simulations table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneySimulationsModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `business_transaction_code` varchar(10) NULL,
    `payment_amount` decimal(15,2) NULL,
    `effective_annual_percentage_rate` decimal(15,2) NULL,
    `administration_fee_amount` decimal(15,2) NULL,
    `total_cost` decimal(15,2) NULL,
    `down_payment_amount` decimal(15,2) NULL,
    `nominal_annual_percentage_rate` decimal(15,2) NULL,
    `first_instalment_date` date NOT NULL,
    `last_instalment_amount` decimal(15,2) NULL,
    `last_insurance_amount` decimal(15,2) NULL,
    `last_interest_amount` decimal(15,2) NULL,
    `last_instalment_number` int(2) NULL,
    `last_collection_date` date NULL,
    `country_code` varchar(2) NULL,
    `shop_id` int(10) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

// Create orders table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `id_order` int(10) NOT NULL,
    `reference` varchar(10) NULL,
    `amount` decimal(15,2) NULL,
    `country_code` varchar(2) NULL,
    `shop_id` int(10) NOT NULL,
    `initial_state` tinyint(1) NOT NULL DEFAULT 1,
    `final_state` tinyint(1) NOT NULL DEFAULT 0,
    `date_add` datetime NOT NULL DEFAULT NOW(),
    `date_upd` datetime NOT NULL DEFAULT NOW(),
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

// Create refunds table
$sql[] = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . ThreeByFourOneyRefundsModel::$definition['table'] . '` (
    `id` int(10) unsigned NOT NULL auto_increment,
    `order_id` int(10) NOT NULL,
    `amount` decimal(15,2) NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8 ;';

foreach ($sql as &$query) {
    if (Db::getInstance()->execute($query) == false) {
        return false;
    }
}

// For upgrades : Update DB if incomplete table previously existed
try {
    Db::getInstance()->execute('ALTER TABLE `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` ADD `shop_id` INT(10) NOT NULL DEFAULT 1;');
    Db::getInstance()->execute('ALTER TABLE `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` ADD `initial_state` tinyint(1) NOT NULL DEFAULT 1');
    Db::getInstance()->execute('ALTER TABLE `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` ADD `final_state` tinyint(1) NOT NULL DEFAULT 0');
    Db::getInstance()->execute('ALTER TABLE `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` ADD `date_add` datetime NOT NULL DEFAULT NOW()');
    Db::getInstance()->execute('ALTER TABLE `' . _DB_PREFIX_ . ThreeByFourOneyOrdersModel::$definition['table'] . '` ADD `date_upd` datetime NOT NULL DEFAULT NOW()');
} catch (PrestaShopDatabaseException $e) {
    // Table already up to date : Do nothing
}
