<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 *
 * @author AXLERATION <contact@axleration.fr>
 * @copyright AXLERATION
 * @license https://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * @category Payment
 */

if (!defined('_PS_VERSION_')) exit;

require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyUtility.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/helpers/ThreeByFourOneyLogger.php';

require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCarriersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyCredentialsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyBusinessTransactionsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneySimulationsModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyOrdersModel.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/models/ThreeByFourOneyRefundsModel.php';

require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyMarketingWebService.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/classes/ThreeByFourOneyPaymentWebService.php';

require_once _PS_MODULE_DIR_ . 'threebyfouroney/controllers/admin/AdminThreeByFourOneyCancellationController.php';
require_once _PS_MODULE_DIR_ . 'threebyfouroney/controllers/admin/AdminThreeByFourOneyConfirmationController.php';

class Threebyfouroney extends PaymentModule
{
    protected $utility;
    protected $carriers;
    protected $credentials;
    protected $business_transactions;
    protected $marketing_web_service;
    protected $payment_web_service;
    protected $logger;
    protected $paymentActive;
    /**
     * __construct function
     * Define global variables and call dependencies.
     * 
     * @version 3.0.0
     * @return mixed
     */
    public function __construct()
    {
        $this->name = 'threebyfouroney';
        $this->tab = 'payments_gateways';
        $this->version = '3.1.18';
        $this->author = 'AXLERATION';
        $this->need_instance = 0;
        $this->bootstrap = true;
        parent::__construct();
        $this->displayName = $this->l('3x4x Oney');
        $this->description = $this->l('The first payment solution in 3 or 4 times by credit card.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall this module?');
        $this->ps_versions_compliancy = array('min' => '1.6.1', 'max' => _PS_VERSION_);
        $this->utility = new ThreeByFourOneyUtility();
        $this->carriers = new ThreeByFourOneyCarriersModel();
        $this->credentials = new ThreeByFourOneyCredentialsModel();
        $this->business_transactions = new ThreeByFourOneyBusinessTransactionsModel();
        $this->marketing_web_service = new ThreeByFourOneyMarketingWebService();
        $this->payment_web_service = new ThreeByFourOneyPaymentWebService();
        $this->logger = new ThreeByFourOneyLogger();
    }

    /**
     * install function
     * install add configuration default values from ps_configuration table, execute /sql/install.php scripts and register required hooks.
     *
     * @version 3.0.0
     * @return void
     */
    public function install()
    {
        if (extension_loaded('curl') == false)
        {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module.');
            return false;
        }

        $psShops = Shop::getShops();
        foreach ($psShops as &$shop)
        {
            $this->utility->installConfiguration($shop['id_shop']);
        }

        include(dirname(__FILE__) . '/sql/install.php');

        $this->utility->installTabs();
        $this->utility->installOrderStatuses();

        $hookNames = array(
            'header',
            'displayBeforeBodyClosingTag',
            'actionAdminControllerSetMedia',
            'displayHome',
            'displayLeftColumn',
            'displayRightColumn',
            'displayProductPriceBlock',
            'displayShoppingCartFooter',
            'paymentOptions',
            'displayPayment',
            'displayPaymentTop',
            'actionOrderHistoryAddAfter',
            'actionObjectCarrierAddAfter',
            'actionObjectShopAddAfter',
            'actionObjectOrderSlipAddAfter'
        );

        if ($check = parent::install()) {
            foreach ($hookNames as $hookName)
                $check &= $this->registerHook($hookName);
        }

        return $check;
    }

    /**
     * uninstall function
     * uninstall removes configuration values from ps_configuration table and execute /sql/uninstall.php scripts.
     *
     * @version 3.0.0
     * @return void
     */
    public function uninstall()
    {
        $this->utility->uninstallConfiguration();
        $this->utility->deleteLegalFile(_PS_MODULE_DIR_ . 'threebyfouroney/uploads/documents/pt/', Configuration::get('TBFONEY_PT_LEGAL_DOCUMENT'));

        include(dirname(__FILE__) . '/sql/uninstall.php');
        $this->utility->uninstallTabs();
        return parent::uninstall();
    }

    /**
     * getContent function
     * getContent defines behaviours expected within module's configuration space.
     *
     * @version 3.0.0|3.0.5|3.1.0
     * @return void
     */
    public function getContent()
    {
        $this->paymentActive = $this->active;
        // Wizard navigation to next step
        if (Tools::getValue('submitWizardNextStep', false))
        {
            $this->wizardNavigateToNextStep();
            if ((bool)Configuration::get('TBFONEY_CARRIERS_CONFIGURED') == false)
            {
                $this->postProcessSaveCarriers();
            }
        }
        // Wizard navigation to previous step
        if (Tools::getValue('submitWizardPreviousStep', false))
        {
            $this->wizardNavigateToPreviousStep();
        }
        // Credentials configuration form (delete method)
        if (Tools::getValue('submitDeleteCredentials', false))
        {
            $this->postProcessDeleteCredentials();
        }
        // Credentials configuration form (save method)
        if (Tools::getValue('submitSaveCredentials', false))
        {
            $this->postProcessSaveCredentials();
        }
        // Store configuration form
        if (Tools::getValue('submitSaveStoreAndOrders', false))
        {
            $this->postProcessSaveStoreAndOrders();
            $this->postProcessSaveCarriers();
            if (isset($_FILES['TBFONEY_PT_LEGAL_DOCUMENT']['name']) && (!empty($_FILES['TBFONEY_PT_LEGAL_DOCUMENT']['name'])) && (!empty($_FILES['TBFONEY_PT_LEGAL_DOCUMENT']['tmp_name'])))
            {
                $alerts = $this->uploadLegalFile();
                $this->context->smarty->assign('alerts', $alerts);
            }
        }
        // Submit save Advanced configuration step form
        if (Tools::getValue('submitSaveAdvanced', false))
        {
            $this->postProcessSaveAdvanced();
        }
        // Submit save Business transactions from database
        if (Tools::getValue('submitSaveBusinessTransactions', false))
        {
            $this->postProcessSaveBusinessTransactions();
        }
        
        $wizardBuilder = $this->wizardConfiguration();
        $wizardConfig = (int)Configuration::get('TBFONEY_WIZARD_STEP');
        $moduleVersion = $this->version;
        // configuration fields for prerequisites step
        $isTermsAcceptedConfig = (bool)Configuration::get('TBFONEY_IS_TERMS_ACCEPTED');
        // configuration fields for Oney\'s credentials step
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $shopId = (int)Context::getContext()->shop->id;
        $langId = (int)Context::getContext()->employee->id_lang;
        // Countries management for configuration
        $countries = $this->utility->countriesMapper();
        $automaticConfirmationConfig = (string)Configuration::get('TBFONEY_AUTOMATIC_CONFIRMATION');
        $storeCategoryConfig = (int)Configuration::get('TBFONEY_STORE_CATEGORY');
        // configuration fields for store step
        $horizontalBannerConfig = (string)Configuration::get('TBFONEY_HORIZONTAL_BANNER');
        $verticalBannerConfig = (string)Configuration::get('TBFONEY_VERTICAL_BANNER');
        $psCarriers = Carrier::getCarriers($langId, true, false, false, false, null, ALL_CARRIERS);
        $psStores = $this->carriers->getStores($langId);
        $ptIsSpecificCountryValidatedConfig = (bool)Configuration::get('TBFONEY_PT_IS_CONFIG_VALIDATED');
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        // configuration fields for advanced step
        $activateLogsConfig = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS');
        $logsLifetimeConfig = (int)Configuration::get('TBFONEY_LOGS_LIFETIME');
        if (_PS_VERSION_ < '1.7.0')
        {
            $isMultistoreActive = (bool)Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE');
        } else {
            $isMultistoreActive = Shop::isFeatureActive();
        }

        $this->logger->deleteLogFilesOlderThan(_PS_MODULE_DIR_ . 'threebyfouroney/logs/', $logsLifetimeConfig);

        $this->context->smarty->assign([
            'token'                                         => Tools::getValue('token'),
            'moduleDir'                                     => $this->_path,
            'formUri'                                       => $_SERVER['REQUEST_URI'],
            'wizardBuilder'                                 => $wizardBuilder,
            'wizardConfig'                                  => $wizardConfig,
            'moduleVersion'                                 => $moduleVersion,
            'checkPhpVersion'                               => $this->utility->checkPhpVersion(),
            'checkPrestashopVersion'                        => $this->utility->checkPrestashopVersion(),
            'checkCurlExtension'                            => $this->utility->checkCurlExtension(),
            'isTermsAcceptedConfig'                         => $isTermsAcceptedConfig,
            'checkCredentialsValidity'                      => $this->utility->checkCredentialsValidity($environmentConfig),
            'checkSpecifiCountriesValidity'                 => $ptIsSpecificCountryValidatedConfig ? 'true' : 'false',
            'environmentConfig'                             => $environmentConfig,
            'adminThreeByFourOneyControllerURI'             => $this->context->link->getAdminLink('AdminThreeByFourOney'),
            'matchedCountries'                              => $countries,
            'automaticConfirmationConfig'                   => $automaticConfirmationConfig,
            'storeCategoryConfig'                           => $storeCategoryConfig,
            'activeCarriers'                                => $psCarriers,
            'deliveryOptions'                               => $this->deliveryOptionsConfiguration(),
            'priorityDeliveries'                            => $this->priorityDeliveriesConfiguration(),
            'addressTypes'                                  => $this->addressTypesConfiguration(),
            'storeCategories'                               => $this->storeCategoriesConfiguration(),
            'activeStores'                                  => $psStores,
            'ptIsCreditIntermediaryConfig'                  => $ptIsCreditIntermediaryConfig,
            'ptIsExclusiveRegimeConfig'                     => $ptIsExclusiveRegimeConfig,
            'ptCompanyLegalDesignationConfig'               => $ptCompanyLegalDesignationConfig,
            'ptLegalDocument'                               => $this->utility->getLegalFile(),
            'horizontalBannerConfig'                        => $horizontalBannerConfig,
            'verticalBannerConfig'                          => $verticalBannerConfig,
            'activateLogsConfig'                            => $activateLogsConfig,
            'logsLifetimeConfig'                            => $logsLifetimeConfig,
            'logsFiles'                                     => $this->logger->scanLogFilesDirectory(_PS_MODULE_DIR_  . $this->name . '/logs/'),
            'isMultistoreActive'                            => $isMultistoreActive
        ]);

        foreach ($countries as &$country)
        {
            $credentials = $this->credentials->getCredentials($environmentConfig, $country['iso_code'], $shopId);
            $businessTransactionsRecords = $this->business_transactions->getBusinessTransactions($country['iso_code'], $shopId);
            if ($credentials || $businessTransactionsRecords)
            {
                // Return all records
                foreach ($credentials as &$credential)
                {
                    $credentialsCountryCodes[] = $credential['country_code'];
                    $this->context->smarty->assign([
                        'TBFONEY_MERCHANT_GUID_' . $country['iso_code']         => $credential['merchant_guid'],
                        'TBFONEY_PSP_GUID_' . $country['iso_code']              => $credential['psp_guid'],
                        'TBFONEY_MARKETING_API_KEY_' . $country['iso_code']     => $credential['marketing_api_key'],
                        'TBFONEY_PAYMENT_API_KEY_' . $country['iso_code']       => $credential['payment_api_key'],
                        'businessTransactionsRecords' . $country['iso_code']    => $businessTransactionsRecords,
                        'credentialsCountryCodes'                               => $credentialsCountryCodes,
                        'currencyConfig' . $country['iso_code']                 => $this->utility->getConfigByLanguage('currency_code', strtolower($country['iso_code']))
                    ]);

                    if ($credential['country_code'] === 'PT')
                    {
                        // Specific country configuration values
                        $this->utility->installSpecificCountryConfiguration($credential['country_code'], $shopId);
                    }
                }
            }
        }

        foreach ($psCarriers as &$carrier)
        {
            $carriersRecords = $this->carriers->getCarriersWithParams($carrier['id_carrier'], $shopId);
            if ($carriersRecords)
            {
                foreach ($carriersRecords as &$record) {
                    $this->context->smarty->assign([
                        'TBFONEY_ADDRESS_TYPE_' . $carrier['id_carrier']                => $record['address_type'],
                        'TBFONEY_DELIVERY_MODE_' . $carrier['id_carrier']               => $record['delivery_mode'],
                        'TBFONEY_DELIVERY_OPTION_' . $carrier['id_carrier']             => $record['delivery_option'],
                        'TBFONEY_PRIORITY_DELIVERY_' . $carrier['id_carrier']           => $record['priority_delivery'],
                        'TBFONEY_ASSOCIATED_STORE_' . $carrier['id_carrier']            => $record['store_id']
                    ]);
                }
            }
        }

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/wizard.tpl');
        return $output;
    }

    /**
     * wizardNavigateToNextStep function
     * wizardNavigateToNextStep defines behaviours expected when the user clicks on "Next step" button.
     * This action modify TBFONEY_WIZARD_STEP and TBFONEY_IS_TERMS_ACCEPTED configuration values
     *
     * @version 3.0.0
     * @return void
     */
    protected function wizardNavigateToNextStep()
    {
        $wizardSteps = $this->wizardConfiguration();
        $firstWizardStep = reset($wizardSteps);
        $lastWizardStep = end($wizardSteps);
        $configurationStep = (int)Configuration::get('TBFONEY_WIZARD_STEP');
        $moduleTermsAccepted = (bool)Configuration::get('TBFONEY_IS_TERMS_ACCEPTED');

        // 1. Accept module terms
        if ($configurationStep == $firstWizardStep || $moduleTermsAccepted != true)
        {
            (bool)Configuration::updateValue('TBFONEY_IS_TERMS_ACCEPTED', 1);
        }
        // 2. Increment to the next step
        if ($configurationStep < (int)$lastWizardStep['number'])
        {
            $incrementStep = $configurationStep += 1;
            (int)Configuration::updateValue('TBFONEY_WIZARD_STEP', $incrementStep);
        }

    }

    /**
     * wizardNavigateToPreviousStep function
     * wizardNavigateToPreviousStep defines behaviours expected when the user clicks on "Previous step" button.
     * This action modify TBFONEY_WIZARD_STEP configuration values
     *
     * @version 3.0.0
     * @return void
     */
    protected function wizardNavigateToPreviousStep()
    {
        $wizardSteps = $this->wizardConfiguration();
        $firstWizardStep = reset($wizardSteps);
        $configurationStep = (int)Configuration::get('TBFONEY_WIZARD_STEP');
        // Decrement to the next step
        if ($configurationStep > (int)$firstWizardStep['number'])
        {
            $decrementStep = $configurationStep -= 1;
            Configuration::updateValue('TBFONEY_WIZARD_STEP', $decrementStep);
        }
    }

    /**
     * postProcessDeleteCredentials function
     * postProcessDeleteCredentials allows users to delete all credentials stored in module's configuration
     * threebyfouroney_credentials table is concerned.
     *
     * @version 3.0.0|3.0.5
     * @return void
     */
    protected function postProcessDeleteCredentials()
    {
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $shopId = (int)Context::getContext()->shop->id;
        $countries = $this->utility->countriesMapper();
        foreach ($countries as &$country)
        {
            $this->credentials->deleteCredentials($environmentConfig, $country['iso_code'], $shopId);
        }
        $this->business_transactions->deleteBusinessTransactions($shopId);

        $alerts = $this->alertsHelper(
            $this->l('Credentials deleted!'),
            $this->l('All credentials has successfully been deleted.'),
            'success'
        );
        $this->context->smarty->assign('alerts', $alerts);
    }

    /**
     * postProcessSaveCredentials function
     * postProcessSaveCredentials defines behaviours expected "Step 2 - Oney's credentials" form submission.
     * Update "Oney's credentials" module's configuration values.
     *
     * @version 3.0.0|3.0.2|3.0.5
     * @return void
     */
    protected function postProcessSaveCredentials()
    {
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $shopId = (int)Context::getContext()->shop->id;
        // Countries management for configuration
        $countries = $this->utility->countriesMapper();
        foreach ($countries as &$country)
        {
            $data = [
                'merchant_guid'     => (string)Tools::getValue('TBFONEY_MERCHANT_GUID_' . $country['iso_code'] . '', (string)Configuration::get('TBFONEY_MERCHANT_GUID_' . $country['iso_code'] . '')),
                'psp_guid'          => (string)Tools::getValue('TBFONEY_PSP_GUID_' . $country['iso_code'] . '', (string)Configuration::get('TBFONEY_PSP_GUID_' . $country['iso_code'] . '')),
                'marketing_api_key' => (string)Tools::getValue('TBFONEY_MARKETING_API_KEY_' . $country['iso_code'] . '', (string)Configuration::get('TBFONEY_MARKETING_API_KEY_' . $country['iso_code'] . '')),
                'payment_api_key'   => (string)Tools::getValue('TBFONEY_PAYMENT_API_KEY_' . $country['iso_code'] . '', (string)Configuration::get('TBFONEY_PAYMENT_API_KEY_' . $country['iso_code'] . '')),
                'is_valid'          => (int)0,
                'shop_id'           => $shopId
            ];
            
            if (!empty($data['merchant_guid']) || !empty($data['psp_guid']) || !empty($data['marketing_api_key']) || !empty($data['payment_api_key']))
            {
                
                // Verify credentials
                $credentialsMarketingWebService = $this->marketing_web_service->verifyCredentials($environmentConfig, $data, $country['iso_code'], $shopId);
                $reference = Order::generateReference();
                $credentialsPaymentWebService = $this->payment_web_service->verifyCredentials($environmentConfig, $data, $reference, $country['iso_code'], $shopId);
                $marketingResponseCodes = (int)$credentialsMarketingWebService['responseCode'];
                $paymentResponseCodes = (int)$credentialsPaymentWebService;
                $responseCodeArray = [$marketingResponseCodes, $paymentResponseCodes];
                if (($marketingResponseCodes != null) && ($paymentResponseCodes != null))
                {
                    switch ($responseCodeArray)
                    {
                        // ONLY for payment API: Concider "OK" if payment response code return 400.
                        case [200, 400]:
                            $isValid = ['is_valid' => 1];
                            $data = array_replace($data, $isValid);
                            $this->credentials->updateCredentials($environmentConfig, $country['iso_code'], $data);
                            $alerts = $this->alertsHelper(
                                $this->l("Credentials configuration saved."),
                                $this->l("Your configuration has successfully been saved and verified."),
                                'success'
                            );
                            $this->context->smarty->assign('alerts', $alerts);
                            // Record credentials
                            $credential = $this->credentials->getCredentials($environmentConfig, $country['iso_code'], $shopId);
                            if (empty($credential))
                            {
                                $this->credentials->insertCredentials($environmentConfig, $country['iso_code'], $data);
                            } else {
                                $this->credentials->updateCredentials($environmentConfig, $country['iso_code'], $data);
                            }
                            break;
                        // Mapping all exeptions cases.
                        case [200, 401]:
                        case [400, 400]:
                        case [401, 400]:
                        case [401, 401]:
                            $alerts = $this->alertsHelper(
                                $this->l("Credentials configuration was not saved."),
                                $this->l("The credentials you entered for " . $country['name'] . " are not correct. Please verify your credentials and try again."),
                                'warning'
                            );
                            return $this->context->smarty->assign('alerts', $alerts);
                            break;
                        default:
                            $alerts = $this->alertsHelper(
                                $this->l("An error occured."),
                                $this->l("Internal server error. Please try again later. If the problem persists contact Oney's administrator for more information about service statuses."),
                                'danger'
                            );
                            return $this->context->smarty->assign('alerts', $alerts);
                    }
                }
            }
        }
    }

    /**
     * postProcessSaveCarriers function
     * Save carriers values from Store and orders step's form
     *
     * @version 3.0.0
     * @return array
     */
    protected function postProcessSaveCarriers()
    {
        $langId = Context::getContext()->employee->id_lang;
        $psCarriers = Carrier::getCarriers($langId, true, false, false, false, null, ALL_CARRIERS);
        $shopId = (int)Context::getContext()->shop->id;
        foreach ($psCarriers as $carrier)
        {
            $data = [
                'address_type'         => ((int)Tools::getValue('TBFONEY_ADDRESS_TYPE_' . $carrier['id_carrier']) != 0) ? (int)Tools::getValue('TBFONEY_ADDRESS_TYPE_' . $carrier['id_carrier'], (int)Configuration::get('TBFONEY_ADDRESS_TYPE_' . $carrier['id_carrier'])) : 4,
                'delivery_mode'        => ((int)Tools::getValue('TBFONEY_DELIVERY_MODE_' . $carrier['id_carrier']) != 0) ? (int)Tools::getValue('TBFONEY_DELIVERY_MODE_' . $carrier['id_carrier'], (int)Configuration::get('TBFONEY_DELIVERY_MODE_' . $carrier['id_carrier'])) : 4,
                'delivery_option'      => ((int)Tools::getValue('TBFONEY_DELIVERY_OPTION_' . $carrier['id_carrier']) != 0) ? (int)Tools::getValue('TBFONEY_DELIVERY_OPTION_' . $carrier['id_carrier'], (int)Configuration::get('TBFONEY_DELIVERY_OPTION_' . $carrier['id_carrier'])) : 2,
                'priority_delivery'    => ((int)Tools::getValue('TBFONEY_PRIORITY_DELIVERY_' . $carrier['id_carrier']) != 0) ? (int)Tools::getValue('TBFONEY_PRIORITY_DELIVERY_' . $carrier['id_carrier'], (int)Configuration::get('TBFONEY_PRIORITY_DELIVERY_' . $carrier['id_carrier'])) : 4,
                'store_id'             => ((int)Tools::getValue('TBFONEY_ASSOCIATED_STORE_' . $carrier['id_carrier']) != 0) ? (int)Tools::getValue('TBFONEY_ASSOCIATED_STORE_' . $carrier['id_carrier'], (int)Configuration::get('TBFONEY_ASSOCIATED_STORE_' . $carrier['id_carrier'])) : 0,
                'shop_id'              => $shopId
            ];

            $carriersRecord = $this->carriers->getCarriersWithParams($carrier['id_carrier'], $shopId);
            $configuredCarriersConfig = (bool)Configuration::get('TBFONEY_CARRIERS_CONFIGURED', null, null, $shopId, false);
            if (empty($carriersRecord) || ($configuredCarriersConfig == false))
            {
                $this->carriers->insertCarriers($carrier['id_carrier'], $data);
                Configuration::updateValue('TBFONEY_CARRIERS_CONFIGURED', 1);
            } else {
                $this->carriers->updateCarriers($carrier['id_carrier'], $data);
            }
        }
    }

    /**
     * getStoreAndOrdersFormValues function
     * getStoreAndOrdersFormValues defines available values in the configuration form.
     *
     * @version 3.0.0
     * @return array
     */
    protected function getStoreAndOrdersFormValues()
    {
        $data["TBFONEY_AUTOMATIC_CONFIRMATION"] = (string)Tools::getValue('TBFONEY_AUTOMATIC_CONFIRMATION', (string)Configuration::get('TBFONEY_AUTOMATIC_CONFIRMATION'));
        $data["TBFONEY_STORE_CATEGORY"] = (string)Tools::getValue('TBFONEY_STORE_CATEGORY', (string)Configuration::get('TBFONEY_STORE_CATEGORY'));
        $data["TBFONEY_HORIZONTAL_BANNER"] = (string)Tools::getValue('TBFONEY_HORIZONTAL_BANNER', (string)Configuration::get('TBFONEY_HORIZONTAL_BANNER'));
        $data["TBFONEY_VERTICAL_BANNER"] = (string)Tools::getValue('TBFONEY_VERTICAL_BANNER', (string)Configuration::get('TBFONEY_VERTICAL_BANNER'));
        $data["TBFONEY_PT_IS_CREDIT_INTERMEDIARY"] = (string)Tools::getValue('TBFONEY_PT_IS_CREDIT_INTERMEDIARY', (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY'));
        $data["TBFONEY_PT_IS_EXCLUSIVITY_REGIME"] = (string)Tools::getValue('TBFONEY_PT_IS_EXCLUSIVITY_REGIME', (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME'));
        $data["TBFONEY_PT_COMPANY_LEGAL_DESIGNATION"] = (string)Tools::getValue('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION', (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION'));

        return $data;
    }

    /**
     * postProcessSaveStoreAndOrders function
     * postProcessSaveStoreAndOrders defines behaviours expected "Step 3 - Store configuration" form submission.
     * Update "Store configuration" module's configuration values.
     *
     * @version 3.0.0
     * @property array $storeValues
     * @return void
     */
    protected function postProcessSaveStoreAndOrders()
    {
        $storeValues = $this->getStoreAndOrdersFormValues();

        foreach (array_keys($storeValues) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }

        $alerts = $this->alertsHelper(
            $this->l('Store configuration saved!'),
            $this->l('Your configuration has successfully been saved.'),
            'success'
        );
        $this->context->smarty->assign('alerts', $alerts);

    }

    /**
     * getAdvancedFormValues function
     * getAdvancedFormValues defines available values in the configuration form.
     *
     * @version 3.0.0
     * @return array
     */
    protected function getAdvancedFormValues()
    {
        $data["TBFONEY_ACTIVATE_LOGS"] = (string)Tools::getValue('TBFONEY_ACTIVATE_LOGS', (string)Configuration::get('TBFONEY_ACTIVATE_LOGS'));
        $data["TBFONEY_LOGS_LIFETIME"] = (int)Tools::getValue('TBFONEY_LOGS_LIFETIME', (int)Configuration::get('TBFONEY_LOGS_LIFETIME'));

        return $data;
    }

    /**
     * postProcessSaveAdvanced function
     * postProcessSaveAdvanced defines behaviours expected "Step 4 - Advanced configuration" form submission.
     * Update "Advanced configuration" module's configuration values.
     *
     * @version 3.0.0
     * @property array $advancedConfigurationValues
     * @return void
     */
    protected function postProcessSaveAdvanced()
    {
        $advancedConfigurationValues = $this->getAdvancedFormValues();

        foreach (array_keys($advancedConfigurationValues) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }

        $alerts = $this->alertsHelper(
            $this->l('Advanced configuration saved!'),
            $this->l('Your configuration has successfully been saved.'),
            'success'
        );
        $this->context->smarty->assign('alerts', $alerts);
    }

    /**
     * postProcessSaveBusinessTransactions function
     * postProcessSaveBusinessTransactions defines behaviours expected "Step 5 - Business transactions" form submission.
     * The only manipulation available for business transactions is activate/inactivate them.
     *
     * @version 3.0.0|3.0.5
     * @return void
     */
    protected function postProcessSaveBusinessTransactions()
    {
        $countries = $this->utility->countriesMapper();
        $shopId = (int)Context::getContext()->shop->id;
        foreach ($countries as &$country)
        {
            $formValues = Tools::getValue('TBFONEY_BUSINESS_TRANSACTION_' . strtoupper($country['iso_code']));
            $formArray = [];
            $businessTransactionsRecords = $this->business_transactions->getBusinessTransactions($country['iso_code'], $shopId);
            $recordsArray = [];
            if ($businessTransactionsRecords != null)
            {
                foreach ($businessTransactionsRecords as &$businessTransaction)
                {
                    $data = [
                        'business_transaction_code' => $businessTransaction['business_transaction_code'],
                        'is_active' => 0
                    ];
                    $formArray[] = $data;
                }
            }
            if (!empty($formValues))
            {
                foreach ($formValues as &$businessTransactionCode)
                {
                    $data = [
                        'business_transaction_code' => $businessTransactionCode,
                        'is_active' => 1
                    ];
                    $recordsArray[] = $data;
                }
            }
            $results = array_merge($formArray, $recordsArray);
            if (!empty($results))
            {
                foreach ($results as &$result)
                {
                    $this->business_transactions->updateBusinessTransactions($result, $country['iso_code'], $shopId);
                }
            }
        }

        $alerts = $this->alertsHelper(
            $this->l('Business transactions saved!'),
            $this->l('Your configuration has successfully been saved.'),
            'success'
        );
        $this->context->smarty->assign('alerts', $alerts);
    }

    public function hookActionAdminControllerSetMedia()
    {
        // parent::setMedia();
        if ((Tools::getValue('controller') == 'AdminModules') && (Tools::getValue('configure') == $this->name))
        {
            $this->context->controller->addCSS($this->_path . 'views/css/threebyfouroney.min.css');
            $this->context->controller->addJS($this->_path . 'views/js/threebyfouroney.min.js');
        }
    }

    /**
     * hookHeader function
     * Dedicated CSS and JavaScript files for the store front-end
     *
     * @version 3.0.0
     * @return void
     */
    public function hookHeader()
    {
       
        if ((Tools::getValue('controller') == 'index') || (Tools::getValue('controller') == 'category') || (Tools::getValue('controller') == 'product') || (Tools::getValue('controller') == 'order') || (Tools::getValue('controller') == 'cart'))
        {
            $this->context->controller->addCSS($this->_path . 'views/css/threebyfouroney.min.css');
            $this->context->controller->addJS($this->_path . 'views/js/threebyfouroney.min.js');
        }
    }


    /**
     * hookDisplayBeforeBodyClosingTag function
     * Place pedagogic popin elements before HTML closing tag.
     *
     * @version 3.0.5
     * @return void
     */
    public function hookDisplayBeforeBodyClosingTag()
    {
        if ((Tools::getValue('controller') == 'index') || (Tools::getValue('controller') == 'category') || (Tools::getValue('controller') == 'product') || (Tools::getValue('controller') == 'order') || (Tools::getValue('controller') == 'cart'))
        {
            $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
            $shopId = (int)Context::getContext()->shop->id;
            $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalments(strtoupper($customerCurrentCountryCode), $shopId);
            $freeBusinessTransactionValues = array_column($activeBusinessTransactionsInstalments, 'free_business_transaction');
            $numberOfInstalmentsArray = array_column($activeBusinessTransactionsInstalments, 'minimum_number_of_instalments');
            
            $this->context->smarty->assign([
                'moduleDir'             => $this->_path,
                'isFreeTransaction'     => !in_array(false, $freeBusinessTransactionValues),
                'numberOfInstalments'   => $numberOfInstalmentsArray
            ]);
            $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/pedagogic_popin.tpl');
            return $output;
            
        }
    }

    /**
     * hookDisplayHome function
     * hookDisplayHome defines behaviours expected on home page content.
     * The behaviour intend to show/hide Oney's horizontal banner regarding module's configuration and active/inactive business transactions.
     *
     * @version 3.0.0|3.0.5
     * @return string
     */
    public function hookDisplayHome()
    {
        if (Configuration::get('PS_CATALOG_MODE')) return;
        if (!$this->utility->currencyDisplayCondition()) return;
        $horizontalBannerConfig = (string)Configuration::get('TBFONEY_HORIZONTAL_BANNER');
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        $ptLegalDocument = $this->utility->getLegalFile();
        if ($horizontalBannerConfig === 'show')
        {
            $customerCurrentLanguageCode = $this->context->language->iso_code;
            $shopId = (int)Context::getContext()->shop->id;
            $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
            $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalments(strtoupper($customerCurrentCountryCode), $shopId);
            if (isset($activeBusinessTransactionsInstalments) && !empty($activeBusinessTransactionsInstalments))
            {
                // Search values in free_business_transaction column
                $freeBusinessTransactionValues = array_column($activeBusinessTransactionsInstalments, 'free_business_transaction');
                // Search only free_business_transaction == false. Only business transaction with fees applied
                $isFreeTransaction = !in_array(false, $freeBusinessTransactionValues);
                $numberOfInstalmentsArray = array_column($activeBusinessTransactionsInstalments, 'minimum_number_of_instalments');
                foreach($numberOfInstalmentsArray as $instalment) $numberOfInstalments = (int)$instalment;

                $this->context->smarty->assign([
                    'moduleDir'                                             => $this->_path,
                    'activeBusinessTransactionsInstalments'                 => $activeBusinessTransactionsInstalments,
                    'isFreeTransaction'                                     => $isFreeTransaction,
                    'numberOfInstalments'                                   => $numberOfInstalments,
                    'customerCurrentCountryCode'                            => $customerCurrentCountryCode,
                    'customerCurrentLanguageCode'                           => $customerCurrentLanguageCode,
                    'ptIsCreditIntermediaryConfig'                          => $ptIsCreditIntermediaryConfig,
                    'ptIsExclusiveRegimeConfig'                             => $ptIsExclusiveRegimeConfig,
                    'ptCompanyLegalDesignationConfig'                       => $ptCompanyLegalDesignationConfig,
                    'ptLegalDocument'                                       => $ptLegalDocument
                ]);
                $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/horizontal_banner.tpl');
                return $output;
            }
        }
    }

    /**
     * hookDisplayLeftColumn function
     * hookDisplayLeftColumn defines behaviours expected on catalog's left column.
     * The behaviour intend to show/hide Oney's vertical banner regarding module's configuration and active/inactive business transactions.
     *
     * @version 3.0.0|3.0.5
     * @return string
     */
    public function hookDisplayLeftColumn()
    {
        if (Configuration::get('PS_CATALOG_MODE')) return;
        if (!$this->utility->currencyDisplayCondition()) return;
        $verticalBannerConfig = (string)Configuration::get('TBFONEY_VERTICAL_BANNER');
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        if ($verticalBannerConfig === 'show')
        {
            $customerCurrentLanguageCode = $this->context->language->iso_code;
            $shopId = (int)Context::getContext()->shop->id;
            $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
            $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalments(strtoupper($customerCurrentCountryCode), $shopId);
            $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
            $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
            $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
            $ptLegalDocument = $this->utility->getLegalFile();
            if (isset($activeBusinessTransactionsInstalments) && !empty($activeBusinessTransactionsInstalments))
            {
                // Search values in free_business_transaction column
                $freeBusinessTransactionValues = array_column($activeBusinessTransactionsInstalments, 'free_business_transaction');
                // Search only free_business_transaction == false. Only business transaction with fees applied
                $searchPaidBusinessTransactions = in_array(false, $freeBusinessTransactionValues);
                // Define $isFreeTransaction value based on paid bt values found or not.
                $searchPaidBusinessTransactions == true ? $isFreeTransaction = false : $isFreeTransaction = true;
                $numberOfInstalmentsArray = array_column($activeBusinessTransactionsInstalments, 'minimum_number_of_instalments');
                foreach($numberOfInstalmentsArray as $instalment) $numberOfInstalments = (int)$instalment;
                $this->context->smarty->assign([
                    'moduleDir'                                 => $this->_path,
                    'activeBusinessTransactionsInstalments'     => $activeBusinessTransactionsInstalments,
                    'isFreeTransaction'                         => $isFreeTransaction,
                    'numberOfInstalments'                       => $numberOfInstalments,
                    'customerCurrentCountryCode'                => $customerCurrentCountryCode,
                    'customerCurrentLanguageCode'               => $customerCurrentLanguageCode,
                    'ptIsCreditIntermediaryConfig'              => $ptIsCreditIntermediaryConfig,
                    'ptIsExclusiveRegimeConfig'                 => $ptIsExclusiveRegimeConfig,
                    'ptCompanyLegalDesignationConfig'           => $ptCompanyLegalDesignationConfig,
                    'ptLegalDocument'                           => $ptLegalDocument
                ]);
                $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/vertical_banner.tpl');
                return $output;
            }
        }
    }

    /**
     * hookDisplayRightColumn function
     * hookDisplayRightColumn defines behaviours expected on catalog's right column.
     * The behaviour is copied from hookDisplayLeftColumn.
     *
     * @version 3.0.0
     * @return void
     */
    public function hookDisplayRightColumn() 
    {    
        return $this->hookDisplayLeftColumn();
    }

    /**
     * hookDisplayProductPriceBlock function
     * hookDisplayProductPriceBlock defines behaviours expected on product page.
     * The behaviour intend to show/hide Oney's simulation regarding product price.
     *
     * @version 3.0.0|3.0.5
     * @param object $params
     * @return void
     */
    public function hookDisplayProductPriceBlock($params)
    {
        if (!$this->utility->currencyDisplayCondition()) return;
        // Place the template only after_price display
        if ($params['type'] === 'after_price')
        {
            $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
            $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
            $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
            $ptLegalDocument = $this->utility->getLegalFile();
            $customerCurrentLanguageCode = $this->context->language->iso_code;
            $shopId = (int)Context::getContext()->shop->id;
            $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
            $productId = (int)Tools::getValue('id_product');
            $productPrice = Product::getPriceStatic($productId);
            $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalmentWithFilter($productPrice, strtoupper($customerCurrentCountryCode), $shopId);
            $activeBusinessTransactionsCodes = array_column($activeBusinessTransactionsInstalments, 'business_transaction_code');
            // Search values in free_business_transaction column
            $freeBusinessTransactionValues = array_column($activeBusinessTransactionsInstalments, 'free_business_transaction');
            // Search only free_business_transaction == false. Only business transaction with fees applied
            $searchPaidBusinessTransactions = in_array(false, $freeBusinessTransactionValues);
            // Define $isFreeTransaction value based on paid bt values found or not.
            $searchPaidBusinessTransactions == true ? $isFreeTransaction = false : $isFreeTransaction = true;
            
            $numberOfInstalmentsArray = array_column($activeBusinessTransactionsInstalments, 'minimum_number_of_instalments');
            foreach($numberOfInstalmentsArray as $instalment) $numberOfInstalments = (int)$instalment;

            if (!empty($activeBusinessTransactionsInstalments))
            {
                foreach ($activeBusinessTransactionsInstalments as &$instalment)
                {
                    $minimumRangeValue = $instalment['minimum_selling_price'];
                    $maximumRangeValue = $instalment['maximum_selling_price'];
                    // Compare min value for minimum_selling_price and max value for maximum_selling_pricer to product price and determine if the simulation is displayed 
                    if (($productPrice >= $minimumRangeValue) && ($productPrice <= $maximumRangeValue))
                    {
                        $this->context->smarty->assign([
                            'moduleDir'                                     => $this->_path,
                            'activeBusinessTransactionsInstalments'         => $activeBusinessTransactionsInstalments,
                            'isFreeTransaction'                             => $isFreeTransaction,
                            'numberOfInstalments'                           => $numberOfInstalments,
                            'ptIsCreditIntermediaryConfig'                  => $ptIsCreditIntermediaryConfig,
                            'ptIsExclusiveRegimeConfig'                     => $ptIsExclusiveRegimeConfig,
                            'ptCompanyLegalDesignationConfig'               => $ptCompanyLegalDesignationConfig,
                            'ptLegalDocument'                               => $ptLegalDocument,
                            'frontThreeByFourOneyControllerURL'             => $this->context->link->getModuleLink($this->name, 'simulations'),
                            'paymentAmount'                                 => $productPrice,
                            'customerCurrentCountryCode'                    => $customerCurrentCountryCode,
                            'customerCurrentLanguageCode'                   => $customerCurrentLanguageCode,
                            'activeBusinessTransactionsCodes'               => json_encode($activeBusinessTransactionsCodes)
                        ]);
                        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/product/simulation_sentence.tpl');
                        return $output;
                    }
                }
            }
        }
    }

    /**
     * hookDisplayShoppingCartFooter function
     * hookDisplayShoppingCartFooter defines behaviours expected on cart resume page.
     * The behaviour intend to show/hide Oney's simulation regarding cart amount.
     *
     * @version 3.0.0|3.0.5
     * @param object $params
     * @return void
     **/
    public function hookDisplayShoppingCartFooter($params)
    {
        if (!$this->utility->currencyDisplayCondition()) return;
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        $ptLegalDocument = $this->utility->getLegalFile();
        $customerCurrentLanguageCode = $this->context->language->iso_code;
        $shopId = (int)Context::getContext()->shop->id;
        $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
        $cartTotalPrice = $this->context->cart->getOrderTotal();
        $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalmentWithFilter($cartTotalPrice, strtoupper($customerCurrentCountryCode), $shopId);
        $activeBusinessTransactionsCodes = array_column($activeBusinessTransactionsInstalments, 'business_transaction_code');
        // Search values in free_business_transaction column
        $freeBusinessTransactionValues = array_column($activeBusinessTransactionsInstalments, 'free_business_transaction');
        // Search only free_business_transaction == false. Only business transaction with fees applied
        $searchPaidBusinessTransactions = in_array(false, $freeBusinessTransactionValues);
        // Define $isFreeTransaction value based on paid bt values found or not.
        $searchPaidBusinessTransactions == true ? $isFreeTransaction = false : $isFreeTransaction = true;
        $numberOfInstalmentsArray = array_column($activeBusinessTransactionsInstalments, 'minimum_number_of_instalments');
        foreach($numberOfInstalmentsArray as $instalment) $numberOfInstalments = (int)$instalment;
        
        if (!empty($activeBusinessTransactionsInstalments))
        {
            foreach ($activeBusinessTransactionsInstalments as &$instalment)
            {
                $checkMinMaxValues = true;
                if (_PS_VERSION_ < '1.7.0')
                {
                    // Compare min value for minimum_selling_price and max value for maximum_selling_pricer to product price and determine if the simulation is displayed
                    $minimumRangeValue = $instalment['minimum_selling_price'];
                    $maximumRangeValue = $instalment['maximum_selling_price'];
                    if (($cartTotalPrice < $minimumRangeValue) && ($cartTotalPrice > $maximumRangeValue)) $checkMinMaxValues = false;
                }
                // Compare min value for minimum_selling_price and max value for maximum_selling_pricer to product price and determine if the simulation is displayed 
                if ($checkMinMaxValues == true)
                {
                    $this->context->smarty->assign([
                        'moduleDir'                                     => $this->_path,
                        'activeBusinessTransactionsInstalments'         => json_encode($activeBusinessTransactionsInstalments),
                        'isFreeTransaction'                             => $isFreeTransaction,
                        'numberOfInstalments'                           => $numberOfInstalments,
                        'ptIsCreditIntermediaryConfig'                  => $ptIsCreditIntermediaryConfig,
                        'ptIsExclusiveRegimeConfig'                     => $ptIsExclusiveRegimeConfig,
                        'ptCompanyLegalDesignationConfig'               => $ptCompanyLegalDesignationConfig,
                        'ptLegalDocument'                               => $ptLegalDocument,
                        'frontThreeByFourOneyControllerURL'             => $this->context->link->getModuleLink($this->name, 'simulations'),
                        'paymentAmount'                                 => $cartTotalPrice,
                        'customerCurrentCountryCode'                    => $customerCurrentCountryCode,
                        'customerCurrentLanguageCode'                   => $customerCurrentLanguageCode,
                        'activeBusinessTransactionsCodes'               => json_encode($activeBusinessTransactionsCodes)
                    ]);
                    $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/cart/simulation_sentence.tpl');
                    return $output;
                }
            }
        }
    }

    /**
     * hookDisplayPaymentTop function
     * hookDisplayPaymentTop defines behaviours expected on cart payment methods selections page.
     * The behaviour intend to show/hide Oney's checkout cartridge regarding cart amount and end customer's information (such as phone number, delivery/shipping address, zip code, country code).
     *
     * @version 3.0.0
     * @return void
     */
    public function hookDisplayPaymentTop()
    {
        if (!$this->paymentActive) return;
        $cart = $this->context->cart;
        $customerCurrentLanguageCode = $this->context->language->id;
        $customerCurrentBillingAddressId = $cart->id_address_invoice;
        $customerCurrentDeliveryAddressId = $cart->id_address_delivery;
        $displayConditions = $this->paymentDisplayConditions($cart, $customerCurrentLanguageCode, $customerCurrentBillingAddressId, $customerCurrentDeliveryAddressId);
        $isNotDisplayable = in_array(false, $displayConditions['is_displayable']);
        if ($isNotDisplayable)
        {
            $message = [];
            foreach($displayConditions['message'] as &$message) $message;
            $alerts = $this->alertsHelper(
                null,
                $message,
                'info'
            );
            $this->context->smarty->assign('alerts', $alerts);
            $output = $this->context->smarty->fetch($this->local_path . 'views/templates/front/alerts.tpl');
            return $output;
        } else {
            return;
        }        
    }

    /**
     * hookDisplayPayment function
     * Display payment for Prestashop 1.6 only 
     *
     * @version 3.0.0|3.0.5
     * @return void
     */
    public function hookDisplayPayment()
    {
        if (!$this->active) return;
        if (!empty($this->hookDisplayPaymentTop())) return;
        $this->paymentActive = $this->active;
        $shopId = (int)Context::getContext()->shop->id;
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        $ptLegalDocument = $this->utility->getLegalFile();
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $cartTotalPrice = $this->context->cart->getOrderTotal();
        $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
        $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalmentWithFilter($cartTotalPrice, strtoupper($customerCurrentCountryCode), $shopId);
        if (!empty($activeBusinessTransactionsInstalments))
        {
            $option = [];
            foreach ($activeBusinessTransactionsInstalments as $key => $instalment)
            {
                $simulationWebService = $this->marketing_web_service->requestSimulations($environmentConfig, $cartTotalPrice, $instalment['business_transaction_code'], $customerCurrentCountryCode);
                if ((int)$instalment['minimum_number_of_instalments'] == 1)
                {
                    $paymentLenght = (int)filter_var($instalment['short_label'], FILTER_SANITIZE_NUMBER_INT);
                }
                $option[$key]['business_transaction_code'] = $simulationWebService->business_transaction_details->business_transaction_code;
                $option[$key]['cost'] = (float)$simulationWebService->payment_amount;
                $option[$key]['total_cost'] = (float)$simulationWebService->payment_amount + (float)$simulationWebService->total_cost;
                $option[$key]['fees'] = (float)$simulationWebService->total_cost;
                $option[$key]['annual_percentage_rate'] = (float)$simulationWebService->effective_annual_percentage_rate;
                $option[$key]['annual_nominal_rate'] = (float)$simulationWebService->nominal_annual_percentage_rate;
                $option[$key]['first_instalment_cost'] = (float)$simulationWebService->down_payment_amount;
                $option[$key]['first_following_amount'] = (float)$simulationWebService->instalments[0]->instalment_amount;
                $option[$key]['number_of_instalments'] = (int)count($simulationWebService->instalments);
                $option[$key]['instalments'] = (int)count($simulationWebService->instalments);
                $option[$key]['free_business_transaction'] = (bool)$instalment['free_business_transaction'];  
            }
            $this->context->smarty->assign([
                'moduleDir'                                     => $this->_path,
                'paymentController'                             => $this->context->link->getModuleLink($this->name, 'payment', ['sid' => $shopId], true),
                'options'                                       => $option,
                'customerCurrentCountryCode'                    => $customerCurrentCountryCode,
                'ptIsCreditIntermediaryConfig'                  => $ptIsCreditIntermediaryConfig,
                'ptIsExclusiveRegimeConfig'                     => $ptIsExclusiveRegimeConfig,
                'ptCompanyLegalDesignationConfig'               => $ptCompanyLegalDesignationConfig,
                'ptLegalDocument'                               => $ptLegalDocument,
                'paymentLenght'                                 => $paymentLenght
            ]);
           
        }
        
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/cart/1.6/checkout_cartridge.tpl');
        return $output;
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @version 3.0.0|3.0.5
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions()
    {
        if (!$this->active) return;
        $this->paymentActive = $this->active;
        if (!empty($this->hookDisplayPaymentTop())) return;
        $shopId = (int)Context::getContext()->shop->id;
        $ptIsCreditIntermediaryConfig = (string)Configuration::get('TBFONEY_PT_IS_CREDIT_INTERMEDIARY');
        $ptIsExclusiveRegimeConfig = (string)Configuration::get('TBFONEY_PT_IS_EXCLUSIVITY_REGIME');
        $ptCompanyLegalDesignationConfig = (string)Configuration::get('TBFONEY_PT_COMPANY_LEGAL_DESIGNATION');
        $ptLegalDocument = $this->utility->getLegalFile();
        $environmentConfig = (string)Configuration::get('TBFONEY_ENVIRONMENT');
        $cartTotalPrice = $this->context->cart->getOrderTotal();
        $customerCurrentCountryCode = $this->utility->customerCountryConfiguration();
        $customerCurrentLanguageCode = $this->context->language->iso_code;
        $activeBusinessTransactionsInstalments = $this->business_transactions->getActiveBTInstalmentWithFilter($cartTotalPrice, strtoupper($customerCurrentCountryCode), $shopId);

        if (!empty($activeBusinessTransactionsInstalments))
        {
            $option = [];
            foreach ($activeBusinessTransactionsInstalments as $key => $instalment)
            {
                $simulationWebService = $this->marketing_web_service->requestSimulations($environmentConfig, $cartTotalPrice, $instalment['business_transaction_code'], $customerCurrentCountryCode);
                $webServiceTotalCost = (float)$simulationWebService->payment_amount + (float)$simulationWebService->total_cost;
                $webServiceFees = (float)$simulationWebService->total_cost;
                $annualPercentageRate = (float)$simulationWebService->effective_annual_percentage_rate;
                $annualNominalRate = (float)$simulationWebService->nominal_annual_percentage_rate;
                $webServiceFirstInstalmentCost = (float)$simulationWebService->down_payment_amount;
                $webServiceFirstFollowingInstalmentCost = (float)$simulationWebService->instalments[0]->instalment_amount;
                $webServiceInstalments = (int)count($simulationWebService->instalments);
                $webServiceCustomerLabel = (string)$instalment['customer_label'];
                $isFreeTransaction = (bool) $instalment['free_business_transaction'];
                $paymentLenght = 0;
                if ($instalment['minimum_number_of_instalments'] == 1)
                {
                    preg_match_all('!\d+!', $instalment['short_label'], $paymentLenght);
                    $paymentLenght = $paymentLenght[0];
                }
               
                $this->context->smarty->assign([
                    'moduleDir'                                     => $this->_path,
                    'numberOfInstalments'                           => $instalment['minimum_number_of_instalments'],
                    'cost'                                          => $simulationWebService->payment_amount,
                    'totalCost'                                     => $webServiceTotalCost,
                    'firstInstalmentCost'                           => $webServiceFirstInstalmentCost,
                    'webServiceFirstFollowingInstalmentCost'        => $webServiceFirstFollowingInstalmentCost,
                    'instalments'                                   => $webServiceInstalments,
                    'webServiceFees'                                => $webServiceFees,
                    'annualPercentageRate'                          => number_format($annualPercentageRate, 2),
                    'annualNominalRate'                             => number_format($annualNominalRate, 1),
                    'customerCurrentCountryCode'                    => strtoupper($customerCurrentCountryCode),
                    'activeBusinessTransactionsCodes'               => $instalment['business_transaction_code'],
                    'ptIsCreditIntermediaryConfig'                  => $ptIsCreditIntermediaryConfig,
                    'ptIsExclusiveRegimeConfig'                     => $ptIsExclusiveRegimeConfig,
                    'ptCompanyLegalDesignationConfig'               => $ptCompanyLegalDesignationConfig,
                    'ptLegalDocument'                               => $ptLegalDocument,
                    'isFreeTransaction'                             => $isFreeTransaction,
                    'customerCurrentLanguageCode'                   => $customerCurrentLanguageCode,
                    'paymentLenght'                                 => $paymentLenght
                ]);
                $template = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/cart/1.7/checkout_cartridge.tpl');
                $option[$key] = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
                $option[$key]->setModuleName($this->name)
                        ->setLogo(_MODULE_DIR_ . 'threebyfouroney/views/img/business_transactions/' . $instalment['minimum_number_of_instalments'] . '.png')
                        ->setCallToActionText($webServiceCustomerLabel)
                        ->setAction($this->context->link->getModuleLink($this->name, 'payment', ['sid' => $shopId], true))
                        ->setInputs([
                            [
                                'type' => 'hidden',
                                'name' => 'business_transaction_code',
                                'value' => $instalment['business_transaction_code'],
                            ]
                        ])
                        ->setAdditionalInformation($template);
            }
            return $option;
        }
        
    }

    /**
     * hookActionObjectOrderSlipAddBefore function
     * Specific manipulation for order refunds
     *
     * @version 3.0.4
     * @param object $params
     * @return void
     */
    public function hookActionObjectOrderSlipAddAfter($params)
    {
        $idOrder = (int)$params['object']->id_order;
        $order = new Order($idOrder);

        if ($order->id && ($order->module == $this->name)) {
            $shopId = (int)$order->id_shop;
            $orderRecord = new ThreeByFourOneyOrdersModel();
            $orderRecordTotalAmount = (float)$orderRecord->getOrderById($idOrder, $shopId)['amount'];
            $orderCancellationAmount = number_format($params['object']->amount, 2, '.', '');
            $shippingCancellationAmount = number_format($params['object']->shipping_cost_amount, 2, '.', '');
            $totalCancellationAmount = (float)$orderCancellationAmount + (float)$shippingCancellationAmount;
            $partialRefundStatusId = (int)Configuration::get('PS_CHECKOUT_STATE_PARTIAL_REFUND');
            $refundStatusId = (int)Configuration::get('PS_OS_REFUND');

            $totalRecordedRefund = 0;
            if ($totalCancellationAmount) {
                $cancellation = new AdminThreeByFourOneyCancellationController();
                $cancellation->sendCancellation($idOrder, $totalCancellationAmount);
                $refundsRecord = new ThreeByFourOneyRefundsModel();
                $alreadyRefundedAmountsRecord = $refundsRecord->getRefundsByOrderId($idOrder);
                foreach ($alreadyRefundedAmountsRecord as $refund) $totalRecordedRefund += (float)$refund['amount'];
                if (((float)$totalRecordedRefund < $orderRecordTotalAmount) && ($totalCancellationAmount < $orderRecordTotalAmount)) {
                    $order->setCurrentState($partialRefundStatusId);
                } else {
                    $order->setCurrentState($refundStatusId);
                }
            } else {
                if (_PS_VERSION_ < '1.7.0') {
                    $redirectionUrl = $this->context->link->getAdminLink('AdminOrders') . '&vieworder&id_order=' . $idOrder;
                } else {
                    $redirectionUrl = $this->context->link->getAdminLink('AdminOrders', 1, [], [
                        'id_order' => $idOrder,
                        'vieworder' => 1,
                    ]);
                }
                Tools::redirectAdmin($redirectionUrl);
            }
        }
    }

    /**
     * hookActionObjectShopAddAfter function
     * Handle default configuration when a new shop is created
     * 
     * @version 3.1.0
     * @param object $params
     * @return void
     */

    public function hookActionObjectShopAddAfter($params)
    {
        $newShopId = (int)$params['object']->id;
        $this->utility->installConfiguration($newShopId);
    }

    /**
     * hookActionObjectCarrierAddAfter function
     * Handle new carriers insertion and apply standard delivery values
     * 
     * @version 3.1.0
     * @param object $params
     * @return void
     */
    public function hookActionObjectCarrierAddAfter($params)
    {
        $newCarrierId = (int)$params['object']->id;
        if ($newCarrierId != 0)
        {
            $shopId = (int)Context::getContext()->shop->id;
            $recordedCarriers = $this->carriers->getCarriers($shopId);
            $carrierIdValues = array_column($recordedCarriers, 'carrier_id');
            $searchCarrierRecords = in_array($newCarrierId, $carrierIdValues);
            if ($searchCarrierRecords === false)
            {
                $data = [
                    'address_type'         => 4,
                    'delivery_mode'        => 4,
                    'delivery_option'      => 2,
                    'priority_delivery'    => 4,
                    'store_id'             => 0,
                    'shop_id'              => $shopId
                ];
                $this->carriers->insertCarriers($newCarrierId, $data);
            }
        }
        
    }

    /**
     * hookActionOrderHistoryAddAfter function
     * Listen order status history hook after status is applied
     * 
     * @version 3.0.5
     * @param object $params
     * @return void
     */
    public function hookActionOrderHistoryAddAfter($params)
    {
        $order = new Order($params['order_history']->id_order);
        
        if ($order->id && ($order->module == $this->name))
        {
            $shopId = (int)Context::getContext()->shop->id;
            $activateLogsConfig = (string)Configuration::get('TBFONEY_ACTIVATE_LOGS');
            if ($activateLogsConfig === 'yes')
            {
                $this->logger->info("ORDER CHANGE : $order->reference");
            }

            $paymentAcceptedStatusId = (int)Configuration::get('PS_OS_PAYMENT');
            $readyToDeliverStatusId = (int)Configuration::get('TBFONEY_OS_READYTODELIVER');
            $refundStatusId = (int)Configuration::get('PS_OS_REFUND');
            $cancelledStatusId = (int)Configuration::get('PS_OS_CANCELED');
            $errorStatusId = (int)Configuration::get('PS_OS_ERROR');

            $orderModel = new ThreeByFourOneyOrdersModel();

            // If ready to deliver -> cascade to payment accepted
            if (((int)$order->current_state == $readyToDeliverStatusId))
            {
                if ($activateLogsConfig === 'yes')
                {
                    $this->logger->info("CASCADE TO PAYMENT ACCEPTED : $order->reference");
                }
                $requestConfirmation = new AdminThreeByFourOneyConfirmationController();
                $requestConfirmation->sendConfirmation((int)$order->id);
                $order->setCurrentState($paymentAcceptedStatusId);
                $orderModel->finalize($order->id);
            }

            // If refund or cancelled or payment error -> send cancellation
            if (in_array((int)$order->current_state, [$refundStatusId, $cancelledStatusId, $errorStatusId])) {
                if ($activateLogsConfig === 'yes')
                {
                    $this->logger->info("SEND CANCELLATION : $order->reference");
                }
        
                $requestCancellation = new AdminThreeByFourOneyCancellationController();
                $requestCancellation->sendCancellation((int)$order->id);
            }

            // If refund or payment error => cascade to cancelled
            if (in_array((int)$order->current_state, [$refundStatusId, $errorStatusId])) {
                if ($activateLogsConfig === 'yes')
                {
                    $this->logger->info("CASCADE TO CANCELLED : $order->reference");
                }

                $this->addHistory($order->id, $cancelledStatusId);
                $order->current_state = $cancelledStatusId;
                $order->save();
                $orderModel->finalize($order->id);
                $this->utility->stockAvailableManagement($order, $shopId);
            }
        }
    }

    private function addHistory($orderId, $stateId) {
        $history = new OrderHistory();
        $history->id_order = $orderId;
        $history->id_order_state = $stateId;
        $history->save();
    }

    /**
     * wizardConfiguration function
     * Define steps for administraion panel's wizard.
     *
     * @version 3.0.0
     * @return array
     */
    public function wizardConfiguration()
    {
        $wizardConfigurationArray = [
            [
                'number'    => 1,
                'anchor'    => '#prerequisites',
                'title'     =>  $this->l('Prerequisites'),
                'has_link'  => true
            ],
            [
                'number'    => 2,
                'anchor'    => '#credentials',
                'title'     =>  $this->l('Oney\'s credentials'),
                'has_link'  => false
            ],
            [
                'number'    => 3,
                'anchor'    => '#store',
                'title'     =>  $this->l('Store and orders configuration'),
                'has_link'  => false
            ],
            [
                'number'    => 4,
                'anchor'    => '#advanced',
                'title'     =>  $this->l('Advanced configuration'),
                'has_link'  => false
            ],
            [
                'number'    => 5,
                'anchor'    => '#business_transactions',
                'title'     =>  $this->l('Business transactions'),
                'has_link'  => false
            ]
        ];
        return $wizardConfigurationArray;
    }

    /**
     * storeCategoriesConfiguration function
     * List Oney's stores categories for administration panel configuration.
     *
     * @version 3.0.0
     * @return array
     */
    public function storeCategoriesConfiguration()
    {
        $storeCategoriesArray = [
            1   =>  $this->l('1 - Food and drinks'),
            2   =>  $this->l('2 - Auto and motorcycle'),
            3   =>  $this->l('3 - Culture and entertainment'),
            4   =>  $this->l('4 - Home and garden'),
            5   =>  $this->l('5 - Home appliances'),
            6   =>  $this->l('6 - Bidding and multi purchasing'),
            7   =>  $this->l('7 - Flowers and gifts'),
            8   =>  $this->l('8 - Computers and software'),
            9   =>  $this->l('9 - Health and beauty'),
            10  =>  $this->l('10 - Personal services'),
            11  =>  $this->l('11 - Professional services'),
            12  =>  $this->l('12 - Sport'),
            13  =>  $this->l('13 - Clothing and accessories'),
            14  =>  $this->l('14 - Travel and tourism'),
            15  =>  $this->l('15 - Hifi, photo and video'),
            16  =>  $this->l('16 - Telephone and communication')
        ];
        return $storeCategoriesArray;
    }

    /**
     * deliveryOptionsConfiguration function
     * List Oney's sdelivery options for administration panel configuration.
     *
     * @version 3.0.0
     * @return void
     */
    public function deliveryOptionsConfiguration()
    {
        $deliveryOptionsArray = [
            1 =>  $this->l('1 - Express'),
            2 =>  $this->l('2 - Standard'),
            3 =>  $this->l('3 - Priority')
        ];
        return $deliveryOptionsArray;
    }

    /**
     * priorityDeliveriesConfiguration function
     * List Oney's priority deliveries for administration panel configuration.
     *
     * @version 3.0.0
     * @return void
     */
    public function priorityDeliveriesConfiguration()
    {
        $priorityDeliveriesArray = [
            1 =>  $this->l('1 - Less than or equal to 1 hour'),
            2 =>  $this->l('2 - Greater than 1 hour'),
            3 =>  $this->l('3 - Immediate'),
            4 =>  $this->l('4 - 24/24 7/7')
        ];
        return $priorityDeliveriesArray;
    }

    /**
     * addressTypesConfiguration function
     * List Oney's address types for administration panel configuration.
     *
     * @version 3.0.0
     * @return void
     */
    public function addressTypesConfiguration()
    {
        $addressTypesArray = [
            1 =>  $this->l('1 - Collection of the goods in the merchant store'),
            2 =>  $this->l('2 - Collection in a third-party point (like ups, alveol, etc.)'),
            3 =>  $this->l('3 - Collection in an airport, train station or travel agency'),
            4 =>  $this->l('4 - Customer’s billing address - Carrier (La Poste, Colissimo, UPS, DHL...or any private carrier)'),
            5 =>  $this->l('5 - Customer’s delivery address - Carrier (La Poste, Colissimo, UPS, DHL...or any private carrier)'),
            6 =>  $this->l('6 - Electronic ticket issuance, download, etc.')
        ];
        return $addressTypesArray;
    }

    /**
     * paymentDisplayConditions function
     * Define payment cartridge display on store payment methods selection page.
     *
     * @version 3.0.0
     * @param object $cart
     * @param int $languageId
     * @param int $billingAddressId
     * @param int $shippingAddressId
     * @return array
     */
    public function paymentDisplayConditions($cart, $languageId, $billingAddressId, $shippingAddressId)
    {
        $display = [];
        if ((isset($cart) && $cart != null) && (isset($languageId) && $languageId != null) && (isset($billingAddressId) && $billingAddressId != null) && (isset($shippingAddressId) && $shippingAddressId != null))
        {
            // Billing address configuration
            $billingAddress = new Address((int)$billingAddressId);
            $billingAddressCountry = Country::getIsoById((int)$billingAddress->id_country);
            $billingAddressPhoneNumber = (!empty($billingAddress->phone_mobile)) ? $billingAddress->phone_mobile : $billingAddress->phone;
            // Based on configuration match phone number
            $configurationPhoneRegex = $this->utility->getConfigByLanguage('phone_regex', strtolower($billingAddressCountry));
            $isMatchingPhoneRegex = preg_match($configurationPhoneRegex, $billingAddressPhoneNumber);
            // Test matching between phone number configuration and billing address phone number
            if ($isMatchingPhoneRegex)
            {
                $display['is_displayable'][] = true;
            } else {
                $display['message'][] =  $this->l('Please enter a valid mobile number to use Oney 3x4x payment.');
                $display['is_displayable'][] = false;
            }
            // Based on configuration match zip code
            $billingAddressZipCode = $billingAddress->postcode;
            $configurationZipCodeRegex = $this->utility->getConfigByLanguage('zip_code_regex', strtolower($billingAddressCountry));
            $isMatchingZipCode = preg_match($configurationZipCodeRegex, $billingAddressZipCode);
            // Test matching between zip code configuration and billing address zip code
            if ($isMatchingZipCode)
            {
                $display['is_displayable'][] = true;
            } else {
                $display['message'][] =  $this->l('Please enter a valid zip code to use Oney 3x4x payment.');
                $display['is_displayable'][] = false;
            }

            //Shipping address configuration
            $shippingAddress = new Address((int)$shippingAddressId);
            // Test matching between billing and shipping address
            if ($billingAddress->id_country === $shippingAddress->id_country)
            {
                $display['is_displayable'][] = true;
            } else {
                $display['code'][] = 'billing_shipping_country';
                $display['message'][] =  $this->l('To use Oney 3x4x payment, please select a delivery or billing address in the same country.');
                $display['is_displayable'][] = false;
            }

            // Carriers configuration
            $shopId = (int)Context::getContext()->shop->id;
            $carriers = $this->carriers->getCarriersWithParams((int)$cart->id_carrier, $shopId);
            foreach ($carriers as $carrier)
            {
                if ($carrier['delivery_mode'] == 1 && $carrier['store_id'] != 0)
                {
                    $carrierStore = $this->carriers->getStoreById($carrier['store_id']);
                    $storeCountryId = (int)$carrierStore['id_country'];
                    if ($storeCountryId != 0)
                    {
                        // Test matching between default store country and billing address
                        if ($billingAddress->id_country == $storeCountryId)
                        {
                            $display['is_displayable'][] = true;
                        } else {
                            $display['code'][] = 'billing_store_country';
                            $display['message'][] =  $this->l('To use Oney 3x4x payment, please select a delivery or billing address in the same country.');
                            $display['is_displayable'][] = false;
                        }
                    }
                }
            }

            // Currency configuration
            $currencyConfiguration = $this->utility->getConfigByLanguage('currency_code', strtolower($billingAddressCountry));
            $currency = new Currency((int)$cart->id_currency);
            // Test if configuration currency matches the currency used in cart
            if ($currency->iso_code === $currencyConfiguration)
            {
                $display['is_displayable'][] = true;
            } else {
                $display['message'][] =  $this->l('Select a currency matching your country to use 3x4x payment.');
                $display['is_displayable'][] = false;
            }
        }
        return $display;
    }

    /**
     * uploadLegalFile function
     * Upload Portugal legal document function.
     *
     * @version 3.0.0|3.0.3
     * @return array
     */
    public function uploadLegalFile()
    {
        $fileInfo = $_FILES['TBFONEY_PT_LEGAL_DOCUMENT'];
        if (isset($fileInfo) && ($fileInfo['error'] === UPLOAD_ERR_OK))
        {
            $explodeFileName = explode('.', $fileInfo['name']);
            $fileExtension = strtolower(end($explodeFileName));
            $sanitizeFileName = md5(time() . $fileInfo['name']) . '.' . $fileExtension;
            $allowedfileExtensions = ['pdf'];
            if (in_array($fileExtension, $allowedfileExtensions))
            {
                $destinationDirectory = _PS_MODULE_DIR_ . 'threebyfouroney/uploads/documents/pt/';
                $destinationPath = $destinationDirectory . $sanitizeFileName;
                if (move_uploaded_file($fileInfo['tmp_name'], $destinationPath))
                {
                    Configuration::updateValue('TBFONEY_PT_IS_CONFIG_VALIDATED', 1, null, 0, Context::getContext()->shop->id);
                    $this->utility->deleteLegalFile($destinationDirectory, $sanitizeFileName);
                    Configuration::updateValue('TBFONEY_PT_LEGAL_DOCUMENT', _PS_BASE_URL_ . __PS_BASE_URI__ . 'modules/threebyfouroney/uploads/documents/pt/' . $sanitizeFileName);
                    $alerts = $this->alertsHelper(
                         $this->l('File successfully uploaded.'),
                         $this->l('Your legal document has been uploaded to the following location: /threebyfouroney/uploads/documents/pt/'),
                        'success'
                    );
                } else {
                    $alerts = $this->alertsHelper(
                         $this->l('File not uploaded.'),
                         $this->l('There was some error moving the file to upload directory. Please make sure the upload directory is writable by web server.'),
                        'warning'
                    );
                }
            } else {
                $alerts = $this->alertsHelper(
                     $this->l('File not uploaded.'),
                     $this->l('Allowed file types: ') . implode(',', $allowedfileExtensions),
                    'warning'
                );
            }
        } else {
            $alerts = $this->alertsHelper(
                 $this->l('File not uploaded.'),
                 $this->l('There is some error in the file upload. Please check the following error: ') . $fileInfo['error'],
                'danger'
            );
        }
        return $alerts;
    }

    /**
     * alertsHelper function
     *
     * @version 3.0.0
     * @param string $title
     * @param string $message
     * @param string $class
     * @return array
     */
    public function alertsHelper($title, $message, $class)
    {
        $alert = [
            $class => [
                'title'     => $title,
                'message'   => $message,
                'class'     => $class
            ]
        ];
        return $alert;
    }
}
